defmodule ProptrackerWeb.Features.Contexts.AdvertisementContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}
  alias ProptrackerWeb.AdvertisementController
  import Plug.Test

  @moduledoc """
  Context for advertisement creation and viewing the form, integrating authentication from AuthContext.
  """

  # Given a registered and logged-in user
  given_ ~r/^a registered and logged-in user$/ do
    fn state ->
      # Prepare user registration data
      user_data = %{username: "advert_user", password: "password123", password_confirmation: "password123"}

      # Register the user
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Simulate user login by initializing session
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)

      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn)}
    end
  end

  # When the user clicks on the "New Advertisement" button on the advertisements index page
  when_ ~r/^the user clicks on the "(?<argument_one>[^"]+)" button on the advertisements index page$/ do
    fn state, %{argument_one: button} ->
      conn = state[:conn]

      # Simulate visiting the advertisements index page
      conn = Phoenix.ConnTest.get(conn, "/advertisements")

      # Simulate clicking the "New Advertisement" button which redirects to "/advertisements/new"
      conn = Phoenix.ConnTest.get(conn, "/advertisements/new")

      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the user should see the new advertisement form
  then_ ~r/^the user should see the new advertisement form$/ do
    fn state ->
      conn = state[:conn]

      # Check that the response contains the advertisement form
      assert conn.resp_body =~ "<form"
      assert conn.resp_body =~ "id=\"new_advertisement_form\""

      {:ok, state}
    end
  end

  # And the form should include all required fields from the advertisement model
  then_ ~r/^the form should include fields for title, pictures, description, type, price, square meters, location, rooms, floor, total floors$/ do
    fn state ->
      conn = state[:conn]

      # Ensure all required fields are present in the response body
      required_fields = [
        "advertisement_title",
        "advertisement_pictures",
        "advertisement_description",
        "advertisement_type",
        "advertisement_price",
        "advertisement_square_meters",
        "advertisement_location",
        "advertisement_rooms",
        "advertisement_floor",
        "advertisement_total_floors"
      ]

      Enum.each(required_fields, fn field_id ->
        assert conn.resp_body =~ "id=\"#{field_id}\""
      end)

      {:ok, state}
    end
  end

  # Given a registered and logged-in user creates an advertisement
  given_ ~r/^a registered and logged-in user creates an advertisement with title "(?<title>[^"]+)" and price "(?<price>[^"]+)"$/ do
    fn state, %{title: title, price: price} ->
      conn = state[:conn]
      user = state[:user]

      # Prepare advertisement data
      ad_data = %{
        title: title,
        price: price,
        description: "A nice advertisement",
        type: "rent",
        square_meters: 120,
        location: "Tallinn",
        rooms: 3,
        floor: 2,
        total_floors: 5
      }

      # Simulate form submission to create the advertisement
      conn =
        Phoenix.ConnTest.post(
          conn,
          "/advertisements",
          %{"advertisement" => ad_data}
        )

      # Verify advertisement is created
      ad = Repo.get_by(Advertisement, title: title)
      assert ad
      {:ok, state |> Map.put(:advertisement, ad)}
    end
  end

  # Then the advertisement should be visible in the advertisements list
  then_ ~r/^the advertisement should be visible in the advertisements list$/ do
    fn state ->
      advertisement = state[:advertisement]

      # Check that the advertisement exists in the database
      assert Repo.get(Advertisement, advertisement.id)
      {:ok, state}
    end
  end

  # Then the user should see a message indicating "Advertisement Created Successfully"
  then_ ~r/^the user should see a message indicating "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      conn = state[:conn]

      # Check if the response body contains the success message
      assert conn.resp_body =~ expected_message

      {:ok, state}
    end
  end

  # Given a registered and logged-in user creates an advertisement with missing title and price
  given_ ~r/^a registered and logged-in user creates an advertisement with missing title and price$/ do
    fn state ->
      conn = state[:conn]
      user = state[:user]

      # Prepare advertisement data with missing title and price
      ad_data = %{
        description: "A nice advertisement",
        type: "rent",
        square_meters: 120,
        location: "Tallinn",
        rooms: 3,
        floor: 2,
        total_floors: 5
      }

      # Simulate form submission to create the advertisement (missing title and price)
      conn =
        Phoenix.ConnTest.post(
          conn,
          "/advertisements",
          %{"advertisement" => ad_data}
        )

      # Check if the advertisement was not created due to missing fields
      assert conn.status == 422  # Unprocessable Entity (validation error)

      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the advertisement should not be visible in the advertisements list
  then_ ~r/^the advertisement should not be visible in the advertisements list$/ do
    fn state ->
      conn = state[:conn]

      # Check if advertisement was not created (i.e., not in the database)
      advertisement = Repo.get_by(Advertisement, title: nil)  # Expecting nil or invalid title
      refute advertisement  # Ensure no advertisement with missing data exists

      {:ok, state}
    end
  end

  # And the user should see a message indicating "Advertisement creation failed"
  then_ ~r/^the user should see a message "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      conn = state[:conn]

      # Check if the response body contains the failure message
      assert conn.resp_body =~ expected_message

      {:ok, state}
    end
  end

  ### Edit Advertisement


  # Given a registered and logged-in user with an existing advertisement
  given_ ~r/^a registered and logged-in user with an existing advertisement titled "(?<argument_one>[^"]+)" priced at "(?<argument_two>[^"]+)"  $/ do
    fn state, %{title: title, price: price} ->
      # Prepare user registration data
      user_data = %{username: "edit_user", password: "password123", password_confirmation: "password123"}

      # Register the user
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Simulate user login by initializing session
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)

      # Create an existing advertisement
      ad_data = %{
        user_id: user.id,
        title: title,
        price: price,
        description: "A sample advertisement",
        type: "rent",
        square_meters: 120,
        location: "Tallinn",
        rooms: 3,
        floor: 2,
        total_floors: 5
      }
      {:ok, advertisement} = Repo.insert(%Advertisement{} |> Advertisement.changeset(ad_data))

      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn) |> Map.put(:advertisement, advertisement)}
    end
  end

  # When the user edits the advertisement to change the title and price
  when_ ~r/^the user edits the advertisement to change the title to "(?<argument_one>[^"]+)" and the price to "(?<argument_two>[^"]+)"  $/ do
    fn state, %{new_title: new_title, new_price: new_price} ->
      conn = state[:conn]
      advertisement = state[:advertisement]

      # Prepare updated advertisement data
      updated_ad_data = %{
        "title" => new_title,
        "price" => new_price
      }

      # Simulate form submission to update the advertisement
      conn =
        Phoenix.ConnTest.put(
          conn,
          "/advertisements/#{advertisement.id}",
          %{"advertisement" => updated_ad_data}
        )

      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the advertisement should be updated in the advertisements list
  then_ ~r/^the advertisement should be updated in the advertisements list with the new title "(?<argument_one>[^"]+)" and price "(?<argument_two>[^"]+)"  $/ do
    fn state, %{new_title: new_title, new_price: new_price} ->
      advertisement = state[:advertisement]

      # Verify the advertisement is updated in the database
      updated_ad = Repo.get(Advertisement, advertisement.id)
      assert updated_ad.title == new_title
      assert updated_ad.price == new_price

      {:ok, state}
    end
  end

  # Then the advertisement should retain the original title and price
  then_ ~r/^the advertisement should retain the original title "(?<original_title>[^"]+)" and price "(?<original_price>[^"]+)" in the advertisements list$/ do
    fn state, %{original_title: original_title, original_price: original_price} ->
      advertisement = state[:advertisement]

      # Verify the advertisement remains unchanged in the database
      unchanged_ad = Repo.get(Advertisement, advertisement.id)
      assert unchanged_ad.title == original_title
      assert unchanged_ad.price == original_price

      {:ok, state}
    end
  end

  # And the user should see a message indicating success or failure
  and_ ~r/^the user should see a flash indicating "(?<argument_one>[^"]+)"  $/ do
    fn state, %{message: message} ->
      conn = state[:conn]

      # Verify the response contains the expected message
      assert conn.resp_body =~ message

      {:ok, state}
    end
  end

  ### Edit Advertisement Negative Case:

  # Given a registered and logged-in user with an existing advertisement
  given_ ~r/^a registered and logged-in user with an existing advertisement title "(?<argument_one>[^"]+)" priced at "(?<argument_two>[^"]+)"  $/ do
    fn state, %{title: title, price: price} ->
      # Prepare user registration data
      user_data = %{username: "edit_user", password: "password123", password_confirmation: "password123"}

      # Register the user
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Simulate user login by initializing session
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)

      # Create an existing advertisement
      ad_data = %{
        user_id: user.id,
        title: title,
        price: price,
        description: "A sample advertisement",
        type: "rent",
        square_meters: 120,
        location: "Tallinn",
        rooms: 3,
        floor: 2,
        total_floors: 5
      }
      {:ok, advertisement} = Repo.insert(%Advertisement{} |> Advertisement.changeset(ad_data))

      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn) |> Map.put(:advertisement, advertisement)}
    end
  end

  # When the user tries to edit the advertisement and leaves the title and price empty
  when_ ~r/^the user tries to edit the advertisement and leaves the title and price empty  $/ do
    fn state ->
      conn = state[:conn]
      advertisement = state[:advertisement]

      # Prepare invalid advertisement data
      invalid_ad_data = %{"title" => "", "price" => ""}

      # Simulate form submission with invalid data
      conn =
        Phoenix.ConnTest.put(
          conn,
          "/advertisements/#{advertisement.id}",
          %{"advertisement" => invalid_ad_data}
        )

      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the advertisement should retain the original title and price
  then_ ~r/^the advertisement should retain the original title "(?<argument_one>[^"]+)" and price "(?<argument_two>[^"]+)" in the advertisements list  $/ do
    fn state, %{original_title: original_title, original_price: original_price} ->
      advertisement = state[:advertisement]

      # Verify the advertisement remains unchanged in the database
      unchanged_ad = Repo.get(Advertisement, advertisement.id)
      assert unchanged_ad.title == original_title
      assert unchanged_ad.price == original_price

      {:ok, state}
    end
  end

  # And the user should see a flash "Advertisement update failed"
  and_ ~r/^the user should see a flash "(?<argument_one>[^"]+)"  $/ do
    fn state, %{message: message} ->
      conn = state[:conn]

      # Verify the response contains the expected message
      assert conn.resp_body =~ message

      {:ok, state}
    end
  end

end
