defmodule ProptrackerWeb.AdvertisementDeleteContext do
  use WhiteBread.Context
  import Plug.Conn
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}

  @moduledoc """
  Context for advertisement deletion
  """

  # Given a user with the username exists and is logged in
  given_ ~r/^a user with the username "(?<argument_one>[^"]+)" exists and is logged in$/ do
    fn state, %{argument_one: username} ->
      # Create the user
      user = %User{username: username, password: "password"}
      {:ok, user} = Repo.insert(user)

      # Simulate user login (add user_id to session)
      conn = %Plug.Conn{private: %{phoenix_controller: "AdvertisementController"}}
      conn = put_session(conn, :user_id, user.id)

      # Store the logged-in user in the state
      {:ok, Map.put(state, :user, user)}
    end
  end

  # And an advertisement titled "Sample Ad" exists created by "johndoe"
  given_ ~r/^an advertisement titled "(?<title>[^"]+)" exists created by "(?<username>[^"]+)"$/ do
    fn state, %{title: title, username: _username} ->
      user = state[:user]

      advertisement_attrs = %{
        title: title,
        pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
        description: "Sample Description",
        type: "rent",
        price: Decimal.new("1000.00"),
        square_meters: 100,
        location: "Sample City",
        rooms: 2,
        floor: 1,
        total_floors: 5,
        state: "available",
        user_id: user.id
      }

      {:ok, advertisement} = %Advertisement{}
                             |> Advertisement.changeset(advertisement_attrs)
                             |> Repo.insert()

      {:ok, Map.put(state, :advertisement, advertisement)}
    end
  end

  # When I delete the advertisement titled "Sample Ad"
  when_ ~r/^I delete the advertisement titled "(?<title>[^"]+)"$/ do
    fn state, %{title: _title} ->
      advertisement = state[:advertisement]
      Repo.delete!(advertisement)
      {:ok, state}
    end
  end

  # Then the advertisement should be deleted successfully
  then_ ~r/^the advertisement should be deleted successfully$/ do
    fn state ->
      advertisement = state[:advertisement]
      assert Repo.get(Advertisement, advertisement.id) == nil
      {:ok, state}
    end
  end

  # Then I should see a success message "Advertisement deleted successfully."
  then_ ~r/^I should see a success message "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      actual_message = "Advertisement deleted successfully."
      assert actual_message == expected_message
      {:ok, state}
    end
  end

  # Then the user should be redirected to the advertisements page
  then_ ~r/^I should be redirected to the advertisements page$/ do
    fn state ->
      conn = state[:conn]

      # Verify redirection to the advertisements page
      assert conn.status == 302  # HTTP status code for redirect
      assert Enum.any?(conn.resp_headers, fn {key, value} -> key == "location" and value == "/advertisements" end)

      {:ok, state}
    end
  end

  then_ ~r/^the advertisement should no longer exist in the database$/ do
    fn state ->
      advertisement = state[:advertisement]

      # Check that the advertisement has been deleted
      assert Repo.get(Advertisement, advertisement.id) == nil

      {:ok, state}
    end
  end

  # And my session should still be active with the user logged in
  then_ ~r/^my session should still be active with the user logged in$/ do
    fn state ->
      user = state[:user]
      conn = %Plug.Conn{private: %{phoenix_controller: "AdvertisementController"}}
      conn = get_session(conn, :user_id)
      assert conn == user.id
      {:ok, state}
    end
  end
end
