defmodule ProptrackerWeb.ViewFavoritesContext do
  use WhiteBread.Context
  import Plug.Conn
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}
  import Phoenix.ConnTest
  alias Proptracker.Accounts.Favorite

  @moduledoc """
  Context for testing advertisement favorites functionality.
  """

  # Given a logged-in user has added advertisements to their favorites list
  given_ ~r/^a logged-in user has added advertisements to their favorites list$/ do
    fn state ->
      user = %User{username: "testuser", password: "password"}
      {:ok, user} = Repo.insert(user)

      # Create favorite advertisements for the user
      ad1 = %Advertisement{title: "Ad 1", user_id: user.id}
      ad2 = %Advertisement{title: "Ad 2", user_id: user.id}
      {:ok, _ad1} = Repo.insert(ad1)
      {:ok, _ad2} = Repo.insert(ad2)

      # Simulate user login (add user_id to session)
      conn = %Plug.Conn{private: %{phoenix_controller: "FavoritesController"}}
      conn = put_session(conn, :user_id, user.id)

      {:ok, Map.put(state, :user, user) |> Map.put(:conn, conn)}
    end
  end

  # When the user navigates to the "My Favorites" page
  when_ ~r/^the user navigates to the "My Favorites" page$/ do
    fn state ->
      conn = state[:conn]
      conn = get(conn, "/favorites")

      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then the user should see a list of their favorite advertisements
  then_ ~r/^the user should see a list of their favorite advertisements$/ do
    fn state ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ "Ad 1"
      assert html_response(conn, 200) =~ "Ad 2"
      {:ok, state}
    end
  end

  # Given a logged-in user has no favorite advertisements
  given_ ~r/^a logged-in user has no favorite advertisements$/ do
    fn state ->
      user = %User{username: "testuser", password: "password"}
      {:ok, user} = Repo.insert(user)

      # Simulate user login (add user_id to session)
      conn = %Plug.Conn{private: %{phoenix_controller: "FavoritesController"}}
      conn = put_session(conn, :user_id, user.id)

      {:ok, Map.put(state, :user, user) |> Map.put(:conn, conn)}
    end
  end

  # Then the user should see a message indicating "No favorites yet"
  then_ ~r/^the user should see a message indicating "No favorites yet"$/ do
    fn state ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ "No favorites yet"
      {:ok, state}
    end
  end

  # Given a non-logged-in user tries to access the "My Favorites" page
  given_ ~r/^a non-logged-in user tries to access the "My Favorites" page$/ do
    fn state ->
      conn = %Plug.Conn{}
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # When the user attempts to navigate to the "My Favorites" page
  when_ ~r/^the user attempts to navigate to the "My Favorites" page$/ do
    fn state ->
      conn = state[:conn]
      conn = get(conn, "/favorites")

      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then the user should be redirected to the homepage
  then_ ~r/^the user should be redirected to the homepage$/ do
    fn state ->
      conn = state[:conn]
      assert redirected_to(conn) == "/"
      {:ok, state}
    end
  end

  # And the user should see a message indicating "You must be logged in to view your favorites."
  then_ ~r/^the user should see a message indicating "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      actual_message = get_flash(state[:conn], :error)
      assert actual_message == expected_message
      {:ok, state}
    end
  end
end
