defmodule ProptrackerWeb.Features.Contexts.AuthContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.User
  alias ProptrackerWeb.SessionController
  alias ProptrackerWeb.UserController
  @moduledoc """
  Context for user authentication, registration, login, profile update, and password change.
  """

  # Given a user with valid registration data
  given_ ~r/^a user with valid registration data$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123"}
      {:ok, Map.put(state, :user_data, user_data)}
    end
  end

  # When the user registers with valid data
  when_ ~r/^the user registers with valid data$/ do
    fn state ->
      user_data = state[:user_data]
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # Then the user should be registered successfully
  then_ ~r/^the user should be registered successfully$/ do
    fn state ->
      user = state[:user]
      assert Repo.get(User, user.id)
      {:ok, state}
    end
  end

  # Given a user with invalid registration data
  given_ ~r/^a user with invalid registration data$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "wrongpassword"}
      {:ok, Map.put(state, :user_data, user_data)}
    end
  end

  # When the user attempts to register with invalid data
  when_ ~r/^the user attempts to register with invalid data$/ do
    fn state ->
      user_data = state[:user_data]
      changeset = User.registration_changeset(%User{}, user_data)
      case Repo.insert(changeset) do
        {:ok, _user} -> {:error, "User should not be registered"}
        {:error, changeset} -> {:ok, Map.put(state, :changeset, changeset)}
      end
    end
  end

  # Then the user should receive an error message indicating the invalid data
  then_ ~r/^the user should receive an error message indicating the invalid data$/ do
    fn state ->
      changeset = state[:changeset]
      assert changeset.valid? == false
      assert changeset.errors[:password_confirmation]
      {:ok, state}
    end
  end


  # Given a registered user
  given_ ~r/^a registered user$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # When the user logs in with valid credentials
  when_ ~r/^the user logs in with valid credentials$/ do
    fn state ->
      user = state[:user]
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)
      conn = SessionController.create_user(conn, %{})
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then the user should be redirected after login
  then_ ~r/^the user should be redirected after login$/ do
    fn state ->
      conn = state[:conn]
      assert conn.status == 302
      assert conn.resp_headers |> Enum.any?(fn {key, value} -> key == "location" and value == "/" end)
      {:ok, state}
    end
  end

  # Negative scenario: Given a registered user with invalid login credentials
  given_ ~r/^a user with invalid login credentials$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # When the user attempts to log in with invalid credentials
  when_ ~r/^the user attempts to log in with invalid credentials$/ do
    fn state ->
      user = state[:user]
      credentials = %{username: user.username, password: "wrongpassword"}  # Using incorrect password
      # Assuming the login function returns an error for invalid credentials
      case SessionController.login(credentials) do
        {:ok, conn} -> {:error, "User should not be logged in"}
        {:error, reason} -> {:ok, Map.put(state, :error_reason, reason)}
      end
    end
  end

  # Then the user should receive an error message indicating invalid credentials
  then_ ~r/^the user should receive an error message indicating invalid credentials$/ do
    fn state ->
      error_reason = state[:error_reason]
      assert error_reason == "Invalid credentials"
      {:ok, state}
    end
  end

  # Given a registered and logged-in user with username "testuser" and name "Test User"
  given_ ~r/^a registered and logged-in user with username "(.+)" and name "(.+)"$/ do
    fn username, name, state ->
      user_data = %{username: username, name: name, password: "password123", password_confirmation: "password123"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)
      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn)}
    end
  end

  # When the user navigates to their profile edit page
  when_ ~r/^the user navigates to their profile edit page$/ do
    fn state ->
      conn = state[:conn]
      user = state[:user]
      conn = UserController.edit(conn, %{"id" => user.id})
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # And they update their username to "newtestuser"
  when_ ~r/^they update their username to "(.+)"$/ do
    fn new_username, state ->
      user = state[:user]
      conn = state[:conn]
      user_params = %{username: new_username}
      conn = UserController.update(conn, %{"id" => user.id, "user" => user_params})
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # And they update their name to "New Test User"
  when_ ~r/^they update their name to "(.+)"$/ do
    fn new_name, state ->
      user = state[:user]
      conn = state[:conn]
      user_params = %{name: new_name}
      conn = UserController.update(conn, %{"id" => user.id, "user" => user_params})
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then they should see a message "User updated successfully."
  then_ ~r/^they should see a message "(.+)"$/ do
    fn expected_message, state ->
      conn = state[:conn]
      actual_message = conn.private[:phoenix_flash][:info]
      assert actual_message == expected_message
      {:ok, state}
    end
  end


  # And their username should be updated to "newtestuser" in the database
  then_ ~r/^their username should be updated to "(.+)" in the database$/ do
    fn expected_username, state ->
      user = state[:user]
      assert Repo.get(User, user.id).username == expected_username
      {:ok, state}
    end
  end

  # And their name should be updated to "New Test User" in the database
  then_ ~r/^their name should be updated to "(.+)" in the database$/ do
    fn expected_name, state ->
      user = state[:user]
      assert Repo.get(User, user.id).name == expected_name
      {:ok, state}
    end
  end

  # Given a registered and logged-in user
  given_ ~r/^a registered and logged-in user$/ do
    fn state ->
      # Assuming a valid user is already created and logged in.
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123", name: "Test User"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # When the user attempts to update their profile with invalid data (empty username or name)
  when_ ~r/^the user attempts to update their profile with invalid data \(empty username or name\)$/ do
    fn state ->
      user = state[:user]

      # Invalid data: Empty username and/or name
      invalid_data = %{username: "", name: ""}

      # Create a changeset with invalid data
      changeset = User.profile_changeset(user, invalid_data)

      # Attempt to update the user with invalid data
      case Repo.update(changeset) do
        {:ok, _updated_user} -> {:error, "Profile updated with invalid data"}
        {:error, changeset} -> {:ok, Map.put(state, :error_reason, changeset.errors)}
      end
    end
  end

  # Then the user should see an error message indicating the invalid data
  then_ ~r/^the user should see an error message indicating the invalid data$/ do
    fn state ->
      error_reason = state[:error_reason]

      # Check for appropriate error messages for empty username and name
      assert Enum.any?(error_reason, fn {field, _msg} -> field == :username end)
      assert Enum.any?(error_reason, fn {field, _msg} -> field == :name end)

      {:ok, state}
    end
  end

  # And the database should not be updated with the invalid data
  then_ ~r/^the database should not be updated with the invalid data.$/ do
    fn state ->
      user = state[:user]
      updated_user = Repo.get(User, user.id)

      # Assert that the user data has not changed in the database
      assert updated_user.username == user.username
      assert updated_user.name == user.name

      {:ok, state}
    end
  end

  # Given a registered and logged-in user with a specified username and password
  given_ ~r/^a registered and logged-in user with username "(?<username>[^"]+)" and password "(?<password>[^"]+)"$/ do
    fn state, %{username: username, password: password} ->
      # Prepare registration data
      user_data = %{username: username, password: password, password_confirmation: password}

      # Create and insert the user into the database
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Log the user in by simulating session creation
      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)
      conn = SessionController.create_user(conn, %{})

      # Update state with user and conn
      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn)}
    end
  end


  # When the user navigates to the password change page
  when_ ~r/^the user navigates to the password change page$/ do
    fn state ->
      conn = state[:conn]
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # And they enter their current password "password123"
  when_ ~r/^they enter their current password "(.+)"$/ do
    fn current_password, state ->
      {:ok, Map.put(state, :current_password, current_password)}
    end
  end

  # And they enter a new password "newpassword123"
  when_ ~r/^they enter a new password "(.+)"$/ do
    fn new_password, state ->
      {:ok, Map.put(state, :new_password, new_password)}
    end
  end

  # And they confirm the new password "newpassword123"
  when_ ~r/^they confirm the new password "(.+)"$/ do
    fn confirmed_password, state ->
      new_password = state[:new_password]

      if new_password == confirmed_password do
        user = state[:user]
        changeset = User.changeset(user, %{password: new_password})
        {:ok, _user} = Repo.update(changeset)
        {:ok, state}
      else
        {:error, "Passwords do not match"}
      end
    end
  end

  # Then the user's password should be updated to "newpassword123" in the database
  then_ ~r/^the user's password should be updated to "(.+)" in the database$/ do
    fn expected_password, state ->
      user = state[:user]
      updated_user = Repo.get(User, user.id)
      assert User.verify_password(updated_user, expected_password)
      {:ok, state}
    end
  end

  # Given a registered and logged-in user with username "testuser" and password "password123"
  given_ ~r/^a registered and logged-in user with username "([^"]*)" and password "([^"]*)"$/ do
    fn state, username, password ->
      user_data = %{username: username, password: password, password_confirmation: password}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # When the user enters an incorrect current password
  when_ ~r/^the user enters their current password "([^"]*)"$/ do
    fn state, entered_current_password ->
      user = state[:user]

      # Incorrect password attempt (wrong current password)
      incorrect_password = entered_current_password

      # Simulate checking the password (compare with the actual password)
      if incorrect_password != user.password do
        {:ok, Map.put(state, :error_reason, "Current password is incorrect.")}
      else
        # If password matched, update with new password (this part is not part of the negative scenario)
        new_password_data = %{password: "newpassword123", password_confirmation: "newpassword123"}
        changeset = User.password_changeset(user, new_password_data)
        Repo.update(changeset)
        {:ok, state}
      end
    end
  end

  # Then the user should see an error message "Current password is incorrect."
  then_ ~r/^the user should see an error message "([^"]*)"$/ do
    fn state, error_message ->
      error_reason = state[:error_reason]
      assert error_reason == error_message
      {:ok, state}
    end
  end

  # And the user's password should not be updated in the database
  then_ ~r/^the user's password should not be updated in the database$/ do
    fn state ->
      user = state[:user]
      updated_user = Repo.get(User, user.id)

      # Ensure the password has not changed in the database
      assert updated_user.password == user.password
      {:ok, state}
    end
  end
end
