defmodule ProptrackerWeb.UserDeleteContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.User

  @moduledoc """
  Context for user deletion
  """

  # Given a user with the username exists
  given_ ~r/^a user with the username "(?<username>[^"]+)" exists$/ do
    fn state, %{username: username} ->
      # Create a new user
      user = %User{username: username, password: "password"}
      # Insert user into the database
      {:ok, user} = Repo.insert(user)

      # Add user to the state
      {:ok, Map.put(state, :user, user)}
    end
  end

  # When I delete a user by their username
  when_ ~r/^I delete the user with the username "(?<username>[^"]+)"$/ do
    fn state, %{username: username} ->
      # Fetch the user to delete using the captured username
      user = Repo.get_by!(User, username: username)
      # Delete the user
      Repo.delete!(user)

      # Add deleted user to the state for further assertions
      {:ok, Map.put(state, :deleted_user, user)}
    end
  end

  # Then the user should be deleted successfully
  then_ ~r/^the user should be deleted successfully$/ do
    fn state ->
      deleted_user = state[:deleted_user]

      # Check that the user is no longer in the database
      assert Repo.get(User, deleted_user.id) == nil
      {:ok, state}
    end
  end

  # Then I should see a success message
  then_ ~r/^I should see a success message "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      actual_message = "User deleted successfully." # Example success message
      assert actual_message == expected_message
      {:ok, state}
    end
  end

  # Then I should be redirected to the users page
  then_ ~r/^I should be redirected to the users page$/ do
    fn state ->
      # This is an example, so assuming successful redirection
      # You can add proper checks here based on your application logic
      assert true
      {:ok, state}
    end
  end

  ### Negative case for attempting to delete the logged-in user's account ###

  # Given a user is logged in
  given_ ~r/^I am logged in with the username "(?<username>[^"]+)"$/ do
    fn state, %{username: username} ->
      # Fetch the logged-in user
      user = Repo.get_by!(User, username: username)

      # Add the logged-in user to the state
      {:ok, Map.put(state, :logged_in_user, user)}
    end
  end

  # When I attempt to delete my own account
  when_ ~r/^I attempt to delete my account$/ do
    fn state ->
      user = state[:logged_in_user]

      # Attempt to delete the logged-in user
      # (Assume there is a check that prevents this action)
      error_message = "Account deletion is not allowed while logged in."

      {:ok, Map.put(state, :error_message, error_message)}
    end
  end

  # Then the account should not be deleted
  then_ ~r/^the account should not be deleted$/ do
    fn state ->
      assert state[:error_message] == "Account deletion is not allowed while logged in."
      {:ok, state}
    end
  end

  # Then I should see an error message
  then_ ~r/^I should receive an error message stating "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      actual_message = state[:error_message]
      assert actual_message == expected_message
      {:ok, state}
    end
  end

  # Then I should remain on the current page
  then_ ~r/^I should remain on the current page$/ do
    fn state ->
      # This is an example, assuming no redirection occurs
      assert true
      {:ok, state}
    end
  end
end
