defmodule ProptrackerWeb.OwnAdvertisementShow do
  use WhiteBread.Context
  use ExUnit.Case, async: true
  import Plug.Conn
  import Phoenix.ConnTest
  import Ecto.Query

  alias Proptracker.Repo
  alias Proptracker.Accounts.User
  alias Proptracker.Accounts.Advertisement

  @moduledoc """
  Context for user advertisement-related features in Proptracker application
  """

  # Given a registered and logged-in user with advertisements
  given_ ~r/^I am logged in as a valid user with advertisements$/ do
    fn state ->
      # Prepare user registration data
      user_data = %{username: "user", password: "password123", password_confirmation: "password123"}

      # Register the user
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Create an advertisement for the logged-in user
      advertisement = %Advertisement{
        title: "Advertisement 1",
        description: "Description of ad 1",
        user_id: user.id
      }
      {:ok, _advertisement} = Repo.insert(advertisement)

      # Simulate user login by initializing session
      conn = %Plug.Conn{} |> init_test_session(user_id: user.id)

      # Store the user, advertisement, and connection in the state
      {:ok, state |> Map.put(:user, user) |> Map.put(:advertisement, advertisement) |> Map.put(:conn, conn)}
    end
  end

  # Given the user has at least one advertisement in their account
  given_ ~r/^I have at least one advertisement listed in my account$/ do
    fn state ->
      user = Map.get(state, :user)

      # Ensure the user has at least one advertisement
      advertisements = Repo.all(from a in Advertisement, where: a.user_id == ^user.id)

      # Assert that the user has at least one advertisement
      assert length(advertisements) >= 1

      {:ok, state}
    end
  end

  # When the user visits the advertisements page
  when_ ~r/^I visit the advertisements page$/ do
    fn state ->
      conn = Map.get(state, :conn)

      # Simulate visiting the advertisements page
      conn = get(conn, "/advertisements")

      # Store the updated connection in the state
      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the user should see a list of advertisements they own
  then_ ~r/^I should see a list of advertisements owned by me$/ do
    fn state ->
      advertisement = Map.get(state, :advertisement)

      # Check that the advertisement is displayed on the page
      assert html_response(state.conn, 200) =~ advertisement.title
      assert html_response(state.conn, 200) =~ advertisement.description

      {:ok, state}
    end
  end

  # Given the user is not logged in
  given_ ~r/^I am not logged in$/ do
    fn state ->
      # Simulate a logged-out state by clearing the session
      conn = %Plug.Conn{} |> init_test_session(%{})

      # Store the connection (not logged in) in the state
      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # When the user visits the advertisements page
  when_ ~r/^I visit the advertisement page$/ do
    fn state ->
      conn = Map.get(state, :conn)

      # Simulate visiting the advertisements page
      conn = get(conn, "/advertisements")

      # Store the updated connection in the state
      {:ok, state |> Map.put(:conn, conn)}
    end
  end

  # Then the user should be redirected to the login page
  then_ ~r/^I should be redirected to the login page$/ do
    fn state ->
      # Assert that the response is a redirect (HTTP 302) to the login page
      assert redirected_to(state.conn) == "/login"

      {:ok, state}
    end
  end
  # When I search for advertisements with type and location
  when_ ~r/^I search for advertisements with type "([^"]*)" and location "([^"]*)"$/ do
    fn state, type, location ->
      conn = Map.get(state, :conn)

      # Simulate searching by making a GET request with query params
      conn = get(conn, "/advertisements?type=#{type}&location=#{location}")

      # Store the updated connection in the state
      {:ok, state |> Map.put(:conn, conn) |> Map.put(:search_params, %{type: type, location: location})}
    end
  end

  # Then I should see a list of advertisements of the correct type and location
  then_ ~r/^I should see a list of advertisements of type "([^"]*)" in "([^"]*)"$/ do
    fn state, type, location ->
      conn = Map.get(state, :conn)

      # Get the advertisements from the response body (assuming HTML response)
      advertisements = html_response(conn, 200)

      # Assert that the advertisements include the type and location
      assert advertisements =~ type
      assert advertisements =~ location

      {:ok, state}
    end
  end



  # Given I am logged in as a valid user
  given_ ~r/^I am logged in as a valid user$/ do
    fn state ->
      # Prepare user registration data
      user_data = %{username: "user", password: "password123", password_confirmation: "password123"}

      # Register the user
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      # Simulate user login by initializing session
      conn = %Plug.Conn{} |> init_test_session(user_id: user.id)

      # Store the user and connection in the state
      {:ok, state |> Map.put(:user, user) |> Map.put(:conn, conn)}
    end
  end

  # When I search for advertisements with type and location
  when_ ~r/^I search for advertisements with type "apartment" and location "city center"$/ do
    fn state, type, location ->
      conn = Map.get(state, :conn)

      # Simulate searching by making a GET request with query params
      conn = get(conn, "/advertisements?type=#{type}&location=#{location}")

      # Store the updated connection in the state
      {:ok, state |> Map.put(:conn, conn) |> Map.put(:search_params, %{type: type, location: location})}
    end
  end

  # Then I should see a list of advertisements of the correct type and location
  then_ ~r/^I should see a list of advertisements of type "apartment" in "city center"$/ do
    fn state, type, location ->
      conn = Map.get(state, :conn)

      # Get the advertisements from the response body (assuming HTML response)
      advertisements = html_response(conn, 200)

      # Assert that the advertisements include the type and location
      assert advertisements =~ type
      assert advertisements =~ location

      {:ok, state}
    end
  end




end
