defmodule ProptrackerWeb.AdvertisementShowContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.Advertisement
  alias Proptracker.Accounts.User

  @moduledoc """
  Context for testing advertisement display functionality.
  """

  # Given a valid property ID exists
  given_ ~r/^a valid property ID exists$/ do
    fn state ->
      advertisement = %Advertisement{
        title: "Modern Apartment",
        price: Decimal.new(250_000),
        location: "Downtown",
        square_meters: 100,
        type: "Apartment",
        rooms: 3,
        pictures: ["/adverts/image1.jpg"],
        description: "A beautiful apartment in the city center.",
        state: "available"
      }

      {:ok, advertisement} = Repo.insert(advertisement)
      {:ok, Map.put(state, :advertisement, advertisement)}
    end
  end

  # Given a user tries to access a property that does not exist
  given_ ~r/^a user tries to access a property that does not exist$/ do
    fn state ->
      {:ok, Map.put(state, :invalid_id, 999_999)}
    end
  end

  # Given an unauthorized user attempts to view a property
  given_ ~r/^an unauthorized user attempts to view a property$/ do
    fn state ->
      {:ok, state}
    end
  end

  # When a user opens the property details page
  when_ ~r/^a user opens the property details page$/ do
    fn state ->
      advertisement = state[:advertisement]
      {:ok, Map.put(state, :response, ProptrackerWeb.AdvertisementController.show(%Plug.Conn{}, %{"id" => advertisement.id}))}
    end
  end

  # When they open the property details page
  when_ ~r/^they open the property details page$/ do
    fn state ->
      id = Map.get(state, :invalid_id, 1)
      {:ok, Map.put(state, :response, ProptrackerWeb.AdvertisementController.show(%Plug.Conn{}, %{"id" => id}))}
    end
  end

  # Then the system should display the property details
  then_ ~r/^the system should display the property title, description, location, price, square meters, type, and rooms$/ do
    fn state ->
      advertisement = state[:advertisement]

      assert advertisement.title == "Modern Apartment"
      assert advertisement.location == "Downtown"
      assert advertisement.price == Decimal.new(250_000)
      assert advertisement.square_meters == 100
      assert advertisement.type == "Apartment"
      assert advertisement.rooms == 3
      {:ok, state}
    end
  end

  # Then the system should display the property pictures in a gallery layout
  then_ ~r/^the system should display the property pictures in a gallery layout$/ do
    fn state ->
      advertisement = state[:advertisement]

      assert Enum.count(advertisement.pictures) > 0
      assert Enum.all?(advertisement.pictures, &String.contains?(&1, "/adverts/"))
      {:ok, state}
    end
  end

  # Then the property state should be displayed prominently
  then_ ~r/^the property state should be displayed prominently$/ do
    fn state ->
      advertisement = state[:advertisement]

      assert advertisement.state == "available"
      {:ok, state}
    end
  end

  # Then the system should display a 404 error with a meaningful message
  then_ ~r/^the system should display a 404 error with a meaningful message$/ do
    fn state ->
      response = state[:response]
      assert response.status == 404
      assert response.body =~ "Property not found"
      {:ok, state}
    end
  end

  # Then the system should display a 403 error indicating restricted access
  then_ ~r/^the system should display a 403 error indicating restricted access$/ do
    fn state ->
      {:ok, state}
    end
  end
end
