defmodule ProptrackerWeb.ViewOwnerProfileContext do
  use WhiteBread.Context
  use Hound.Helpers
  use ExUnit.Case, async: true

  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}
  import Ecto.Query

  scenario_starting_state fn state ->
    Hound.start_session()

    clean_database()

    # Insert owner into the database
    owner = %User{name: "testowner", password: "password"}
    {:ok, owner} = Repo.insert(owner)

    # Insert advertisement associated with the owner
    advertisement = %Advertisement{
      title: "Test Advertisement",
      description: "A great property",
      price: Decimal.new("1500.00"),
      square_meters: 80,
      location: "Sample Location",
      rooms: 3,
      floor: 2,
      total_floors: 5,
      user_id: owner.id
    }
    {:ok, ad} = Repo.insert(advertisement)

    # Add the advertisement and owner to the state
    state = Map.put(state, :advertisement, ad) |> Map.put(:owner, owner)
    #IO.inspect(state, label: "Starting state with owner and advertisement")
    state
  end

  scenario_finalize fn _status, _state ->
    Hound.end_session()
    :ok
  end

  # Given: I am on the homepage
  given_ ~r/^I am on the homepage$/, fn state ->
    navigate_to("http://localhost:4000/")
    {:ok, state}
  end

  # When: I am viewing an advertisement that belongs to an owner with active advertisements
  when_ ~r/^I am viewing an advertisement that belongs to an owner with active advertisements$/, fn state ->
    owner = %User{username: "testowner", password: "password"}
    {:ok, owner} = Repo.insert(owner)

    advertisement = %Advertisement{
      title: "Test Advertisement",
      description: "A great property",
      price: Decimal.new("1500.00"),
      square_meters: 80,
      location: "Sample Location",
      rooms: 3,
      floor: 2,
      total_floors: 5,
      user_id: owner.id
    }

    {:ok, ad} = Repo.insert(advertisement)
    state = Map.put(state, :advertisement, ad) |> Map.put(:owner, owner)
    #IO.inspect(state, label: "Updated State")
    {:ok, state}
  end

  # Then: I see a button to view the owner's profile
  then_ ~r/^I see a button to view the owner's profile$/, fn state ->
    assert visible_text({:css, ".owner-button"}) == "Owner's Profile"
    {:ok, state}
  end

  # And: An advertisement has a 'View Profile' button
  and_ ~r/^an advertisement has a 'View Profile' button$/, fn state ->
    assert visible_text({:css, ".owner-button"}) == "Owner's Profile"
    {:ok, state}
  end

 # When: I click on the 'View Profile' button
when_ ~r/^I click on the 'View Profile' button for an advertisement$/, fn state ->
  button = find_element(:css, ".owner-button")
  click(button)
  {:ok, state}
end

# Then: I am redirected to the owner's profile page
then_ ~r/^I am redirected to the owner's profile page$/, fn state ->
  advertisement = state[:advertisement]
  assert advertisement != nil, "Advertisement is nil in state"
  assert current_url() == "http://localhost:4000/advertisements/#{advertisement.id}/owner"

  {:ok, state}
end


  # Given: I am on the owner's profile page
  given_ ~r/^I am on the owner's profile page$/, fn state ->
    advertisement = state[:advertisement]
    assert advertisement != nil, "Advertisement is nil in state"
    navigate_to("http://localhost:4000/advertisements/#{advertisement.id}/owner")
    {:ok, state}
  end

  when_ ~r/^the owner has active advertisements$/, fn state ->
    {:ok, state}
  end

  # Then: I see the owner's profile details
  then_ ~r/^I see the owner's profile details$/, fn state ->
    owner = state[:owner]
    assert visible_text({:css, ".owner-profile"}) =~ owner.name
    {:ok, state}
  end

  # Then: I see a list of the owner's active advertisements
  then_ ~r/^I see a list of the owner's active advertisements$/, fn state ->
    owner = state[:owner]
    advertisements = Repo.all(Advertisement |> where(user_id: ^owner.id))

    Enum.each(advertisements, fn ad ->
      assert visible_text({:css, ".advertisement-title"}) =~ ad.title
    end)

    {:ok, state}
  end

  defp clean_database do
    # Delete all advertisements and users
    Repo.delete_all(Advertisement)
    Repo.delete_all(User)
  end
end
