defmodule ProptrackerWeb.Features.Contexts.RecommendedPropertiesContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.Advertisement
  use Hound.Helpers
  import Ecto.Query


  scenario_starting_state fn state ->
    Hound.start_session()
    clear_advertisements()
    state
  end

  scenario_finalize fn _status, _state ->
    #Hound.end_session()
    :ok
  end

  # Background steps
  given_ ~r/^there are multiple properties in the system$/, fn state ->
    insert_sample_properties()
    {:ok, state}
  end

  and_ ~r/^I am viewing a specific advertisement page$/, fn state ->
    # Select the advertisement with ID = 1
    advertisement = Repo.one(from(a in Advertisement, where: a.id == 1))

    # Ensure the advertisement with ID 1 exists
    if advertisement == nil do
      raise "Advertisement with ID 1 not found. Ensure the test setup inserts the advertisement."
    end

    # Navigate to the advertisement's page
    navigate_to("/advertisements/#{advertisement.id}")
    {:ok, Map.put(state, :advertisement, advertisement)}
  end


  # Positive Case
  when_ ~r/^there are similar properties available$/, fn state ->
    insert_similar_properties()
    {:ok, state}
  end

  then_ ~r/^up to 5 recommended properties are displayed$/, fn state ->
    recommendations = find_all_elements(:css, ".property-box")

    assert length(recommendations) == 5
    {:ok, state}
  end

  and_ ~r/^each recommendation shows key details: price, rooms, surface area, and location$/, fn state ->
    recommendations = find_all_elements(:css, ".property-box")

    for recommendation <- recommendations do
      text = visible_text(recommendation)

      assert text =~ "Price:"
      assert text =~ "Rooms:"
      assert text =~ "Surface Area:"
      assert text =~ "Location:"
    end

    {:ok, state}
  end




  # Negative Case
  when_ ~r/^there are no similar properties available$/, fn state ->
    clear_similar_properties()
    {:ok, state}
  end

  then_ ~r/^a message "No similar properties found" is displayed$/, fn state ->
    recommendations = find_all_elements(:css, ".recommended-container")
    text = visible_text(recommendations)

    assert text =~ "No similar properties found"
    {:ok, state}
  end


  # Edge Case
  when_ ~r/^fewer than 5 similar properties exist$/, fn state ->
    insert_limited_similar_properties(3)
    {:ok, state}
  end

  then_ ~r/^only the available recommended properties are displayed$/, fn state ->
    recommendations = find_all_elements(:css, ".property-box")

    assert length(recommendations) == 3
    {:ok, state}
  end

  # Helper functions
  defp clear_advertisements do
    Repo.delete_all(Advertisement)
  end

  defp insert_sample_properties do
    for i <- 1..10 do
      %Advertisement{
        id: i,
        price: 1500 + i * 1000,
        rooms: 2,
        square_meters: 50 + i * 1,
        location: "Location #{i}",
        pictures: ["/adverts/default_ad.jpg"]
      }
      |> Repo.insert!()
    end
  end

  defp insert_similar_properties do
    for i <- 1..5 do
      %Advertisement{
        price: 2500 + i * 10,
        rooms: 2,
        square_meters: 50,
        location: "Similar Location #{i}",
        pictures: ["/adverts/default_ad.jpg"]
      }
      |> Repo.insert!()
    end
  end

  defp clear_similar_properties do
    Repo.delete_all(from(a in Advertisement, where: a.location == "Similar Location"))
  end

  defp insert_limited_similar_properties(count) do
    for i <- 1..count do
      %Advertisement{
        price: 2500 + i * 10,
        rooms: 2,
        square_meters: 50,
        location: "Limited Location #{i}",
        pictures: ["/adverts/default_ad.jpg"]
      }
      |> Repo.insert!()
    end
  end

  defp element_displayed?(type, selector, opts \\ []) do
    case find_element(type, selector, opts) do
      nil -> false
      _ -> true
    end
  end
end
