defmodule ProptrackerWeb.Features.Contexts.FilterSearchContext do
  use WhiteBread.Context
  import Plug.Conn
  import Phoenix.ConnTest
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}

  # Given I have filled the search form with specific data
  given_ ~r/^I have filled the search form with type "(?<type>[^"]+)", query "(?<query>[^"]+)", location "(?<location>[^"]+)"$/ do
    fn state, %{"type" => type, "query" => query, "location" => location} ->
      # Store the search parameters in the state
      {:ok, Map.put(state, :search_params, %{type: type, query: query, location: location})}
    end
  end

  # Given I have entered a minimum room count of "(?<min_rooms>\d+)" and a maximum room count of "(?<max_rooms>\d+)"
  given_ ~r/^I have entered a minimum room count of "(?<min_rooms>\d+)" and a maximum room count of "(?<max_rooms>\d+)"$/ do
    fn state, %{"min_rooms" => min_rooms, "max_rooms" => max_rooms} ->
      # Update search params with room filters
      search_params = Map.get(state, :search_params)
      updated_params = Map.put(search_params, :min_rooms, String.to_integer(min_rooms))
                              |> Map.put(:max_rooms, String.to_integer(max_rooms))
      {:ok, Map.put(state, :search_params, updated_params)}
    end
  end

  # Given I have entered an invalid room range where max_rooms < min_rooms
  given_ ~r/^I have entered a minimum room count of "(?<min_rooms>\d+)" and a maximum room count of "(?<max_rooms>\d+)" which is invalid$/ do
    fn state, %{"min_rooms" => min_rooms, "max_rooms" => max_rooms} ->
      # Store the invalid room parameters and simulate an error condition
      search_params = Map.get(state, :search_params)
      updated_params = Map.put(search_params, :min_rooms, String.to_integer(min_rooms))
                              |> Map.put(:max_rooms, String.to_integer(max_rooms))

      # Check if max_rooms is less than min_rooms, and if so, add an error to the state
      if String.to_integer(max_rooms) < String.to_integer(min_rooms) do
        {:ok, Map.put(state, :error, "Max rooms cannot be less than min rooms")}
      else
        {:ok, Map.put(state, :search_params, updated_params)}
      end
    end
  end

  # When I submit the search form
  when_ ~r/^I submit the search form$/ do
    fn state ->
      # Simulate submitting the form with the search parameters (including room filters)
      conn = post(state[:conn], "/advertisements", state[:search_params])
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then I should see search results for the given query
  then_ ~r/^I should see search results for "(?<query>[^"]+)"$/ do
    fn state, %{query: query} ->
      conn = state[:conn]
      # Ensure that the response contains the query in the search results
      assert html_response(conn, 200) =~ query
      {:ok, state}
    end
  end

  # And the location should be displayed
  then_ ~r/^the location "(?<location>[^"]+)" should be displayed$/ do
    fn state, %{location: location} ->
      conn = state[:conn]
      # Ensure that the location is included in the search results
      assert html_response(conn, 200) =~ location
      {:ok, state}
    end
  end

  # And the type should be displayed
  then_ ~r/^the type "(?<type>[^"]+)" should be displayed$/ do
    fn state, %{type: type} ->
      conn = state[:conn]
      # Ensure that the type (rent/sell) is included in the search results
      assert html_response(conn, 200) =~ type
      {:ok, state}
    end
  end

  # And the room count should be between the minimum and maximum room count
  then_ ~r/^the room count should be between "(?<min_rooms>\d+)" and "(?<max_rooms>\d+)"$/ do
    fn state, %{min_rooms: min_rooms, max_rooms: max_rooms} ->
      conn = state[:conn]
      # Ensure the search results match the room count filter
      advertisements = html_response(conn, 200) |> Floki.find(".advertisement-room-count")
      Enum.each(advertisements, fn ad ->
        room_count = Floki.text(ad) |> String.to_integer()
        assert room_count >= String.to_integer(min_rooms) and room_count <= String.to_integer(max_rooms)
      end)
      {:ok, state}
    end
  end

  # Then I should see a message indicating no results if no rooms match
  then_ ~r/^I should see a message "(?<message>[^"]+)" if no rooms match$/ do
    fn state, %{message: message} ->
      conn = state[:conn]
      # Ensure the search results show a message for no results found
      assert html_response(conn, 200) =~ message
      {:ok, state}
    end
  end

  # Then I should see an error message indicating that max rooms cannot be less than min rooms
  then_ ~r/^I should see an error message indicating that max rooms cannot be less than min rooms$/ do
    fn state ->
      # Check if an error message exists in the state
      if Map.get(state, :error) do
        assert state[:error] == "Max rooms cannot be less than min rooms"
      end
      {:ok, state}
    end
  end

  # When I click on the reset button
  when_ ~r/^I click on the reset button$/ do
    fn state ->
      # Simulate clicking the reset button
      conn = get(state[:conn], "/advertisements")
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then the search form should be cleared
  then_ ~r/^the search form should be cleared$/ do
    fn state ->
      conn = state[:conn]
      # Ensure that the form is reset to its default state
      assert html_response(conn, 200) =~ "Search by title..."
      assert html_response(conn, 200) =~ "Location..."
      assert html_response(conn, 200) =~ "--Type--"
      {:ok, state}
    end
  end

  # And I should see the default state of the form
  then_ ~r/^I should see the default state of the form$/ do
    fn state ->
      conn = state[:conn]
      # Check that the form has returned to the default empty state
      assert html_response(conn, 200) =~ "Search by title..."
      assert html_response(conn, 200) =~ "Location..."
      assert html_response(conn, 200) =~ "--Type--"
      {:ok, state}
    end
  end

    # Step to enter a minimum room count of 2 and a maximum room count of 4
  and_ ~r/^I have entered a minimum room count of 2 and a maximum room count of 4$/ do
    fn state ->
      # Set the minimum and maximum room count values in the search parameters
      updated_params = Map.put(state[:search_params], :min_rooms, 2)
                                |> Map.put(:max_rooms, 4)
      {:ok, Map.put(state, :search_params, updated_params)}
    end
  end

  # Step to enter a minimum room count of 5 and a maximum room count of 6
  and_ ~r/^I have entered a minimum room count of 5 and a maximum room count of 6$/ do
    fn state ->
      # Set the minimum and maximum room count values in the search parameters
      updated_params = Map.put(state[:search_params], :min_rooms, 5)
                                |> Map.put(:max_rooms, 6)
      {:ok, Map.put(state, :search_params, updated_params)}
    end
  end

  # Step to enter a minimum room count of 4 and a maximum room count of 2 (invalid case)
  and_ ~r/^I have entered a minimum room count of 4 and a maximum room count of 2$/ do
    fn state ->
      # Set the minimum and maximum room count values in the search parameters
      updated_params = Map.put(state[:search_params], :min_rooms, 4)
                                |> Map.put(:max_rooms, 2)
      {:ok, Map.put(state, :search_params, updated_params)}
    end
  end

  then_ ~r/^no advertisements should be displayed$/ do
    fn state ->
      conn = state[:conn]
      html = html_response(conn, 200)

      # Ensure the advertisements section is empty
      if String.contains?(html, "<div class=\"advertisements\">") and
         not String.contains?(html, "<div class=\"advertisement\">") do
        {:ok, state}
      else
        {:error, "Advertisements are displayed despite mismatched filters"}
      end
    end
  end

  then_ ~r/^I should see an error message "(?<error_message>[^"]+)"$/ do
    fn state, %{error_message: error_message} ->
      conn = state[:conn]
      # Ensure the error message is present in the response
      if String.contains?(html_response(conn, 200), error_message) do
        {:ok, state}
      else
        {:error, "Expected error message not found: #{error_message}"}
      end
    end
  end
end
