defmodule ProptrackerWeb.Features.Contexts.PropertySearchContext do
  use WhiteBread.Context
  import Plug.Conn
  import Phoenix.ConnTest

  alias Proptracker.Repo
  alias Proptracker.Accounts.Advertisement

  # Given I am a user on the property search page
  given_ ~r/^I am a user on the property search page$/ do
    fn state ->
      conn = build_conn()
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # And I have searched for properties with certain criteria (e.g., type, location, price)
  given_ ~r/^I have searched for properties with certain criteria \(e\.g\., type, location, price\)$/ do
    fn state ->
      # Define search parameters
      search_params = %{
        "type" => "Apartment",
        "location" => "Tallinn",
        "min_price" => "100000",
        "max_price" => "500000"
      }

      # Simulate submitting the search form
      conn = post(state[:conn], "/property-search", search_params)
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # When the search results are displayed
  when_ ~r/^the search results are displayed$/ do
    fn state ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ "Search Results"
      {:ok, state}
    end
  end

  # Then I should see the available properties listed before the reserved properties
  then_ ~r/^I should see the available properties listed before the reserved properties$/ do
    fn state ->
      conn = state[:conn]
      advertisements = html_response(conn, 200) |> Floki.find(".advertisement-status")

      # Verify ordering: positive test
      availability_ordered = Enum.reduce_while(advertisements, true, fn ad, acc ->
        status = Floki.text(ad)
        cond do
          status == "Available" and acc -> {:cont, true}
          status == "Reserved" -> {:halt, true}
          true -> {:halt, false}
        end
      end)

      assert availability_ordered, "Available properties are not listed before reserved ones"
      {:ok, state}
    end
  end

  # Then I should see an error if reserved properties are listed before available ones (Negative Test)
  then_ ~r/^I should see an error if reserved properties are listed before available ones$/ do
    fn state ->
      conn = state[:conn]
      advertisements = html_response(conn, 200) |> Floki.find(".advertisement-status")

      # Verify ordering: negative test
      availability_not_ordered = Enum.any?(advertisements, fn ad ->
        status = Floki.text(ad)
        status == "Reserved" and Enum.find(advertisements, fn a -> Floki.text(a) == "Available" end)
      end)

      assert not availability_not_ordered, "Reserved properties are incorrectly listed before available ones"
      {:ok, state}
    end
  end
end
