defmodule ProptrackerWeb.FavoritesContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, Advertisement}
  import Phoenix.ConnTest, only: [build_conn: 0, post: 3]

  @moduledoc """
  Context for testing advertisement favorites functionality.
  """

  # Given a registered user
  given_ ~r/^a registered user$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)
      {:ok, Map.put(state, :user, user)}
    end
  end

  # Given a logged-in user is viewing an advertisement
  given_ ~r/^a logged-in user is viewing an advertisement$/ do
    fn state ->
      user = state[:user]

      # Create an advertisement
      advertisement = %Advertisement{
        title: "Test Advertisement",
        price: 1000,
        location: "Test Location",
        description: "Test Description"
      }

      # Insert the advertisement into the database
      {:ok, advertisement} = Repo.insert(advertisement)

      # Store the advertisement in the state for later use
      {:ok, Map.put(state, :advertisement, advertisement)}
    end
  end

  # When the user logs in and the session is set
  when_ ~r/^the user logs in$/ do
    fn state ->
      user = state[:user]
      conn = build_conn()

      # Simulate the session by adding the user to the connection (no `put_session/3`)
      conn = %{conn | assigns: %{user_id: user.id}}

      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # When the user clicks the "Add to Favorites" button
  when_ ~r/^the user clicks the "(?<argument_one>[^"]+)" button$/ do
    fn state ->
      # Retrieve the necessary data from the state
      conn = state[:conn]
      advertisement_id = state[:advertisement].id
      is_favorite = state[:is_favorite]

      # Simulate the button click and make the POST request to toggle the favorite status
      conn = post(conn, "/advertisements/#{advertisement_id}/toggle_favorite", %{is_favorite: is_favorite})

      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then the advertisement should be added to the user's favorites list
  then_ ~r/^the advertisement should be added to the user's favorites list$/ do
    fn state ->
      user = state[:user]
      advertisement = state[:advertisement]

      # Check if the advertisement is in the user's favorite list
      updated_user = Repo.get(User, user.id)
      assert advertisement.id in updated_user.favorite_advertisements

      {:ok, state}
    end
  end

  # Then the advertisement should be removed from the user's favorites list
  then_ ~r/^the advertisement should be removed from the user's favorites list$/ do
    fn state ->
      user = state[:user]
      advertisement = state[:advertisement]

      # Check if the advertisement has been removed from the user's favorite list
      updated_user = Repo.get(User, user.id)
      refute advertisement.id in updated_user.favorite_advertisements

      {:ok, state}
    end
  end

  # Define the scenario for adding/removing from favorites
  given_ ~r/^a logged-in user has already added an advertisement to their favorites list$/ do
    fn state ->
      user = state[:user]
      advertisement = state[:advertisement]

      # Add the advertisement to the user's favorite list
      updated_user = Repo.get(User, user.id)
      updated_favorites = [advertisement.id | updated_user.favorite_advertisements]

      # Update the user's favorite list in the database
      changeset = Ecto.Changeset.change(updated_user, %{favorite_advertisements: updated_favorites})
      Repo.update!(changeset)

      {:ok, Map.put(state, :user, updated_user)}
    end
  end
end
