defmodule ProptrackerWeb.Features.Contexts.PastSearchesContext do
  use WhiteBread.Context
  import Plug.Conn
  import Phoenix.ConnTest
  alias Proptracker.Repo
  alias Proptracker.Accounts.{User, SearchHistory}

  @moduledoc """
  Context for past searches feature.
  """

  # Given a user with the username exists and is logged in
  given_ ~r/^a user with the username "(?<argument_one>[^"]+)" exists and is logged in$/ do
    fn state, %{argument_one: username} ->
      # Create the user
      user = %User{username: username, password: "password"}
      {:ok, user} = Repo.insert(user)

      # Simulate user login (add user_id to session)
      conn = %Plug.Conn{private: %{phoenix_controller: "PastSearchesController"}}
      conn = put_session(conn, :user_id, user.id)

      # Store the logged-in user in the state
      {:ok, Map.put(state, :user, user)}
    end
  end

  # And the user has made a past search with the query "apartment" in "Tallinn" for type "rent"
  given_ ~r/^the user has made a past search with the query "(?<query>[^"]+)" in "(?<location>[^"]+)" for type "(?<type>[^"]+)"$/ do
    fn state, %{query: query, location: location, type: type} ->
      user = state[:user]

      search_attrs = %{
        query: query,
        location: location,
        type: type,
        user_id: user.id
      }

      {:ok, _search_history} = Repo.insert(%SearchHistory{} |> SearchHistory.changeset(search_attrs))

      {:ok, state}
    end
  end

  # When I visit the past searches page
  when_ ~r/^I visit the past searches page$/ do
    fn state ->
      conn = state[:conn]
      conn = get(conn, "/past_searches")

      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then I should see a list of past searches
  then_ ~r/^I should see a list of past searches$/ do
    fn state ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ "Past Searches"
      {:ok, state}
    end
  end

  # And the past search with the query "apartment" should be displayed
  then_ ~r/^the past search with the query "(?<query>[^"]+)" should be displayed$/ do
    fn state, %{query: query} ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ query
      {:ok, state}
    end
  end

  # And the location "Tallinn" should be displayed
  then_ ~r/^the location "(?<location>[^"]+)" should be displayed$/ do
    fn state, %{location: location} ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ location
      {:ok, state}
    end
  end

  # And the type "rent" should be displayed
  then_ ~r/^the type "(?<type>[^"]+)" should be displayed$/ do
    fn state, %{type: type} ->
      conn = state[:conn]
      assert html_response(conn, 200) =~ type
      {:ok, state}
    end
  end

  # And my session should still be active with the user logged in
  then_ ~r/^my session should still be active with the user logged in$/ do
    fn state ->
      user = state[:user]
      conn = state[:conn]
      assert get_session(conn, :user_id) == user.id
      {:ok, state}
    end
  end

  # Given I am not logged in
  given_ ~r/^I am not logged in$/ do
    fn state ->
      conn = %Plug.Conn{}
      {:ok, Map.put(state, :conn, conn)}
    end
  end

  # Then I should be redirected to the login page
  then_ ~r/^I should be redirected to the login page$/ do
    fn state ->
      conn = state[:conn]
      assert redirected_to(conn) == "/login"
      {:ok, state}
    end
  end

  # And I should see a message "You need to log in to access past searches."
  then_ ~r/^I should see a message "(?<message>[^"]+)"$/ do
    fn state, %{message: expected_message} ->
      actual_message = get_flash(state[:conn], :error)
      assert actual_message == expected_message
      {:ok, state}
    end
  end
end
