defmodule ProptrackerWeb.GoogleMapContext do
  use WhiteBread.Context
  alias Proptracker.Repo
  alias Proptracker.Accounts.Advertisement
  alias Proptracker.Accounts.User
  alias ProptrackerWeb.AdvertisementController
  import Phoenix.ConnTest, only: [html_response: 2]

  @moduledoc """
  Context for testing Google Map display functionality on the advertisement show page.
  """

  # Given a valid advertisement with latitude and longitude exists
  given_ ~r/^a valid advertisement with latitude and longitude exists$/ do
    fn state ->
      advertisement = %Advertisement{
        title: "Modern Apartment",
        price: Decimal.new(250_000),
        location: "Downtown",
        square_meters: 100,
        type: "Apartment",
        rooms: 3,
        pictures: ["/adverts/image1.jpg"],
        description: "A beautiful apartment in the city center.",
        state: "available",
        latitude: 40.7128,  # example latitude
        longitude: -74.0060 # example longitude
      }

      {:ok, advertisement} = Repo.insert(advertisement)
      {:ok, Map.put(state, :advertisement, advertisement)}
    end
  end

  # Given a user is logged in
  given_ ~r/^the user is logged in$/ do
    fn state ->
      user_data = %{username: "testuser", password: "password123", password_confirmation: "password123"}
      changeset = User.registration_changeset(%User{}, user_data)
      {:ok, user} = Repo.insert(changeset)

      conn = %Plug.Conn{} |> Plug.Test.init_test_session(user_id: user.id)
      {:ok, Map.put(state, :user, user) |> Map.put(state, :conn, conn)}
    end
  end

  # When the user opens the advertisement show page
  when_ ~r/^the user navigates to the advertisement show page$/ do
    fn state ->
      advertisement = state[:advertisement]
      conn = state[:conn]
      response = AdvertisementController.show(conn, %{"id" => advertisement.id})
      {:ok, Map.put(state, :response, response)}
    end
  end

  # Then the page should display a Google Map
  then_ ~r/^the page should display a Google Map$/ do
    fn state ->
      response = state[:response]
      # Check if the page contains the Google Map
      assert html_response(response, 200) =~ ~r/id="map"/
      {:ok, state}
    end
  end

  # Then the map should be centered at the correct latitude and longitude
  then_ ~r/^the map should be centered at the correct latitude and longitude$/ do
    fn state ->
      advertisement = state[:advertisement]
      response = state[:response]

      # Check for the Google Map element with the correct coordinates
      assert html_response(response, 200) =~ ~r/lat: #{advertisement.latitude}/
      assert html_response(response, 200) =~ ~r/lng: #{advertisement.longitude}/
      {:ok, state}
    end
  end
end
