defmodule Proptracker.Accounts.Advertisement do
  use Ecto.Schema
  import Ecto.Changeset
  import Ecto.Query  # Importing the Ecto.Query module for query building
  alias Proptracker.Repo
  alias Proptracker.Accounts.Advertisement
  alias Proptracker.Accounts.Favorite

  schema "advertisements" do
    field :title, :string
    field :pictures, {:array, :string}  # Storing image paths in an array of strings
    field :description, :string
    field :type, :string
    field :price, :decimal
    field :square_meters, :integer
    field :location, :string
    field :rooms, :integer
    field :floor, :integer
    field :total_floors, :integer
    field :reference, :string
    field :state, :string, default: "available"
    field :interest, :string, default: "interest"
    field :latitude, :float  # Latitude field
    field :longitude, :float  # Longitude field

    belongs_to :user, Proptracker.Accounts.User

    timestamps()

  end

  # Changeset for creating/updating an advertisement
  def changeset(advertisement, attrs) do
    advertisement
    |> cast(attrs, [:title, :pictures, :description, :type, :price, :square_meters, :location, :rooms, :floor, :total_floors, :state, :user_id, :latitude, :longitude])
    |> validate_required([:title, :type, :price, :square_meters, :location, :rooms, :state])
    |> validate_number(:price, greater_than_or_equal_to: 0, message: "price is not neagtive")
    |> validate_number(:square_meters, greater_than_or_equal_to: 0, message: "must be a positive number")
    |> validate_number(:rooms, greater_than_or_equal_to: 0, message: "must be a positive number")
    |> validate_number(:floor, greater_than_or_equal_to: 0, message: "must be a positive number")
    |> validate_number(:total_floors, greater_than_or_equal_to: 0, message: "must be a positive number")
    |> validate_floor_not_greater_than_total_floors()
    |> validate_pictures()
    |> generate_reference()
  end

  defp generate_reference(changeset) do
    if get_field(changeset, :reference) do
      changeset  # Skip if reference is already present
    else
      reference = "REF-#{:crypto.strong_rand_bytes(4) |> Base.encode16()}"
      put_change(changeset, :reference, reference)
    end
  end

  defp validate_floor_not_greater_than_total_floors(changeset) do
    floor = get_field(changeset, :floor)
    total_floors = get_field(changeset, :total_floors)

    if floor && total_floors && floor > total_floors do
      add_error(changeset, :floor, "Floor cannot be greater than total floors")
    else
      changeset
    end
  end


  defp validate_pictures(changeset) do
    pictures = get_field(changeset, :pictures) || []

    # Skip validation if the changeset has no changes to the "pictures" field (e.g., during updates)
    if Map.get(changeset.changes, :pictures) do
      cond do
        Enum.empty?(pictures) ->
          add_error(changeset, :pictures, "At least one picture is required.")

        Enum.any?(pictures, fn path -> not String.ends_with?(path, ~w(.jpg .jpeg .png)) end) ->
          add_error(changeset, :pictures, "Only JPG and PNG files are allowed.")

        true ->
          changeset
      end
    else
      changeset
    end
  end



  def list_advertisements do
    Repo.all(Advertisement)
  end

  # Function to create a new advertisement
  def create_advertisement(user, advertisement_params) do
    %Advertisement{}
    |> Advertisement.changeset(advertisement_params)
    |> Ecto.Changeset.put_change(:user_id, user.id)  # Associate the advertisement with the user
    |> Repo.insert()
  end

  # Function to update an advertisement
  def update_advertisement(advertisement, advertisement_params) do
    advertisement
    |> Advertisement.changeset(advertisement_params)
    |> Repo.update()
  end

  #updating interest
  def update_interest(advertisement, new_interest) do
    advertisement
    |> Ecto.Changeset.change(interest: new_interest)
    |> Repo.update()
  end

  # Function to list all advertisements for a user
  def list_advertisements_by_user(user_id) do
    from(a in Advertisement,
      where: a.user_id == ^user_id,
      order_by: [
        asc: fragment("CASE WHEN interest = 'not interest' THEN 1 ELSE 0 END"),
        desc: fragment("CASE WHEN state = ? THEN 1 ELSE 0 END", "available"),
        asc: a.updated_at
      ],
    )
    |> Repo.all()
  end

  # Function to get an advertisement by ID (or raise error if not found)
  def get_advertisement!(id) do
    Repo.get!(Advertisement,id)
  end

  # For interest
  def get_advertisementInterest!(id) do
    Repo.get!(Advertisement,id)
  end

  def get_recommended_price(advertisement_id, square_meters, rooms, location) do
    query =
      from(a in __MODULE__,
        where: a.square_meters >= ^round(square_meters * 0.9),
        where: a.square_meters <= ^round(square_meters * 1.1),
        where: a.rooms == ^rooms,
        where: a.location == ^location
      )

    query =
      if is_nil(advertisement_id) do
        query
      else
        from(a in query, where: a.id != ^advertisement_id)
      end

    query
    |> select([a], avg(a.price))
    |> Proptracker.Repo.one()
  end


  # Function to delete an advertisement
  def delete_advertisement(advertisement) do
    Repo.delete(advertisement)
  end

  #favorites of the user:
  def update_favorites(%Advertisement{} = advertisement, new_favorites) do
    advertisement
    |> Ecto.Changeset.change(favorites: new_favorites)
    |> Repo.update()
  end

  def get_favorite(user_id, advertisement_id) do
    IO.inspect({user_id, advertisement_id}, label: "Fetching favorite")
    Repo.get_by(Favorite, user_id: user_id, advertisement_id: advertisement_id)
  end

  # Function to get favorite advertisements by user
  def list_favorite_advertisements(user_id) do
    from(a in Advertisement,
      where: a.user_id == ^user_id and a.favorites == true and a.state == "available",
      select: a
    )
    |> Repo.all()
  end

# Similarity functions to calculate similar ads
@high_similarity_threshold "0.2" # This will be used as a string (representing 10%)
@moderate_similarity_rooms 1

def calculate_similarity(advertisement, other_advertisement) do
  # Calculate similarities for price, surface, and rooms
  price_similarity =
    calculate_percentage_similarity(advertisement.price, other_advertisement.price, @high_similarity_threshold)

  surface_similarity =
    calculate_percentage_similarity(advertisement.square_meters, other_advertisement.square_meters, @high_similarity_threshold)

  rooms_similarity = calculate_rooms_similarity(advertisement.rooms, other_advertisement.rooms)

  # Calculate the final similarity score (average of all factors)
  final_score =
    Decimal.add(price_similarity, surface_similarity)
    |> Decimal.add(Decimal.new(rooms_similarity))
    |> Decimal.div(Decimal.new("3"))
    |> Decimal.to_float()

  # Return the final score and individual factor scores
  {final_score, %{price_similarity: Decimal.to_float(price_similarity), surface_similarity: Decimal.to_float(surface_similarity), rooms_similarity: rooms_similarity}}
end

defp calculate_percentage_similarity(value1, value2, threshold_percentage) do
  # Convert values to Decimal for accurate calculations
  value1 = Decimal.new(to_string(value1)) # Convert value1 to string first
  value2 = Decimal.new(to_string(value2)) # Convert value2 to string first
  threshold = Decimal.new(threshold_percentage) # Threshold percentage as string

  # Calculate absolute difference between values
  difference = Decimal.sub(value1, value2) |> Decimal.abs()

  # Calculate threshold based on percentage difference relative to the first value
  threshold_value = Decimal.mult(value1, threshold)

  # If the difference exceeds the threshold, similarity is low
  if Decimal.cmp(difference, threshold_value) == :gt do
    # Directly return 0 if the difference is larger than the threshold
    Decimal.new("0")
  else
    # Calculate percentage similarity: (threshold - difference) / threshold
    percentage = Decimal.div(difference, threshold_value) |> Decimal.mult(Decimal.new("100"))

    # Return similarity as 100 - the calculated percentage difference
    Decimal.sub(Decimal.new("100"), percentage)
  end
end

defp calculate_rooms_similarity(rooms1, rooms2) do
  # Calculate similarity based on room differences
  case abs(rooms1 - rooms2) do
    0 -> 100
    1 -> 70
    _ -> 0
  end
end

def get_similar_advertisements(advertisement, all_advertisements) do
  all_advertisements
  |> Enum.filter(&(&1.id != advertisement.id)) # Exclude the current advertisement
  |> Enum.map(fn other_ad ->
    {similarity, factors} = calculate_similarity(advertisement, other_ad)

    %{
      advertisement: other_ad,
      similarity: similarity,
      factors: factors
    }
  end)
  |> Enum.filter(&(&1.similarity >= 70)) # Filter by similarity threshold
  |> Enum.sort_by(&(&1.similarity), :desc) # Sort by similarity score
  |> Enum.take(5) # Take top 5
end


end
