defmodule Proptracker.Accounts.User do
  use Ecto.Schema
  import Ecto.Changeset
  alias Proptracker.Repo
  import Ecto.Query
  alias Proptracker.Accounts.Favorite

  schema "users" do
    field :name, :string
    field :username, :string
    field :date_of_birth, :date
    field :password, :string
    field :user_details, :string
    field :phone_number, :string
    field :favorite_advertisements, {:array, :integer}, default: []
    field :interested, {:array, :integer}, default: []
    field :not_interested, {:array, :integer}, default: []

    has_many :advertisements, Proptracker.Accounts.Advertisement

    timestamps()
  end

  @spec changeset(
          {map(),
           %{
             optional(atom()) =>
               atom()
               | {:array | :assoc | :embed | :in | :map | :parameterized | :supertype | :try,
                  any()}
           }}
          | %{
              :__struct__ => atom() | %{:__changeset__ => any(), optional(any()) => any()},
              optional(atom()) => any()
            }
        ) :: Ecto.Changeset.t()
  def changeset(struct, params \\ %{}) do
    struct
    |> cast(params, [:name, :username, :date_of_birth, :password,
    :user_details, :phone_number, :favorite_advertisements, :interested,
      :not_interested
    ])
    |> validate_required([:name, :username, :date_of_birth, :password, :phone_number])
    |> validate_username_uniqueness()
    |> validate_phone_number()
    |> validate_date_of_birth()
    |> validate_password()
  end

  defp validate_username_uniqueness(changeset) do
    if changeset |> get_change(:username) do
      username = get_field(changeset, :username)
      user_id = get_field(changeset, :id)

      existing_user_query =
        if is_nil(user_id) do
          from(u in Proptracker.Accounts.User, where: u.username == ^username)
        else
          from(u in Proptracker.Accounts.User, where: u.username == ^username and u.id != ^user_id)
        end

      existing_user = Repo.one(existing_user_query)

      if existing_user do
        add_error(changeset, :username, "has already been taken")
      else
        changeset
      end
    else
      changeset
    end
  end

  defp validate_phone_number(changeset) do
    phone_number = get_field(changeset, :phone_number)

    if phone_number && String.length(phone_number) < 6 do
      add_error(changeset, :phone_number, "must be at least 6 characters long")
    else
      if phone_number && Regex.match?(~r/^\+?[0-9]+$/, phone_number) do
        changeset
      else
        add_error(changeset, :phone_number, "must only contain digits and may start with '+'")
      end
    end
  end

  defp validate_date_of_birth(changeset) do
    date_of_birth = get_field(changeset, :date_of_birth)

    if date_of_birth && Date.compare(date_of_birth, Date.utc_today()) != :lt do
      add_error(changeset, :date_of_birth, "must be in the past")
    else
      changeset
    end
  end

  defp validate_password(changeset) do
    password = get_field(changeset, :password)

    # Check if the password is at least 6 characters long
    if password && String.length(password) < 6 do
      add_error(changeset, :password, "must be at least 6 characters long")
    else
      changeset
    end
  end
end
