defmodule ProptrackerWeb.AdvertisementController do
  use ProptrackerWeb, :controller
  require Logger
  alias Proptracker.Accounts.User
  alias Proptracker.Accounts.Advertisement
  alias Proptracker.Repo
  import Ecto.Query

  # Apply AuthPlug to protect routes
  plug ProptrackerWeb.AuthPlug when action in [:index, :new, :create, :edit, :update, :delete]

  # List all advertisements owned by the current user
  def index(conn, _params) do
    user = conn.assigns.current_user
     # Retrieve the current user if logged in
    user_id = get_session(conn, :user_id)
    current_user = if user_id, do: Repo.get(Proptracker.Accounts.User, user_id), else: nil
    advertisements = Advertisement.list_advertisements_by_user(user.id)
    render(conn, "index.html",
    advertisements: advertisements,
    current_user: current_user,
    type: nil,
    query: "",
    location: ""
    )
  end

  #for search special:
  def user_search(conn, params) do
    user_id = get_session(conn, :user_id)

    # Retrieve the current user if logged in
    current_user = if user_id, do: Repo.get(Proptracker.Accounts.User, user_id), else: nil

    # Ensure a user is logged in
    if current_user do
      type = Map.get(params, "type", "")
      query = Map.get(params, "query", "")
      location = Map.get(params, "location", "")
      state = Map.get(params, "state", "")
      min_price = Map.get(params, "min_price") |> parse_optional_integer()
      max_price = Map.get(params, "max_price") |> parse_optional_integer()
      min_rooms = Map.get(params, "min_rooms") |> parse_optional_integer()
      max_rooms = Map.get(params, "max_rooms") |> parse_optional_integer()

      # Start query with current user's advertisements
      advertisements_query =
        Advertisement
        |> where([a], a.user_id == ^current_user.id)

      # Apply filters dynamically if they are not empty
      advertisements_query =
        advertisements_query
        |> (fn q -> if type != "", do: from(a in q, where: a.type == ^type), else: q end).()
        |> (fn q -> if query != "", do: from(a in q, where: ilike(a.title, ^"%#{query}%")), else: q end).()
        |> (fn q -> if location != "", do: from(a in q, where: ilike(a.location, ^"%#{location}%")), else: q end).()
        |> (fn q -> if state != "", do: from(a in q, where: a.state == ^state), else: q end).()
        |> (fn q -> if min_price, do: from(a in q, where: a.price >= ^min_price), else: q end).()
        |> (fn q -> if max_price, do: from(a in q, where: a.price <= ^max_price), else: q end).()
        |> (fn q -> if min_rooms, do: from(a in q, where: a.rooms >= ^min_rooms), else: q end).()
        |> (fn q -> if max_rooms, do: from(a in q, where: a.rooms <= ^max_rooms), else: q end).()

      # Exclude 'sold/rented' advertisements and order by state
      advertisements_query =
        advertisements_query
        |> where([a], a.state != "sold/rented")
        |> order_by([a], asc: a.state)

      # Select only the required fields, including pictures
      advertisements =
        advertisements_query
        |> select([a], %{
          id: a.id,
          title: a.title,
          price: a.price,
          location: a.location,
          type: a.type,
          rooms: a.rooms,
          state: a.state,
          pictures: a.pictures
        })
        |> Repo.all()
      render(conn, "index.html",
        advertisements: advertisements,
        current_user: current_user,
        location: location,
        query: query,
        type: type
      )
    else
      conn
      |> put_flash(:error, "You must be logged in to view your advertisements.")
      |> redirect(to: ~p"/login")

    end
  end


  # Helper function to parse optional integers
  defp parse_optional_integer(value) do
    case Integer.parse(value || "") do
      {num, _} -> num
      :error -> nil
    end
  end


  # Show form for creating a new advertisement
  def new(conn, _params) do
    changeset = Advertisement.changeset(%Advertisement{}, %{})
    render(conn, "new.html", changeset: changeset)
  end



  # Create a new advertisement
  @spec create(Plug.Conn.t(), map()) :: Plug.Conn.t()
  def create(conn, %{"advertisement" => advertisement_params}) do
    if conn.assigns[:current_user] do
      pictures = process_pictures(Map.get(advertisement_params, "pictures"))
      IO.inspect(pictures, label: "Processed Pictures")
      updated_params = Map.put(advertisement_params, "pictures", pictures)

      case Advertisement.create_advertisement(conn.assigns.current_user, updated_params) do
        {:ok, advertisement} ->
          conn
          |> put_flash(:info, "Advertisement created successfully.")
          |> redirect(to: ~p"/advertisements")

        {:error, changeset} ->
          IO.inspect(changeset.errors, label: "Validation Errors")
          render(conn, "new.html", changeset: changeset)
      end
    else
      conn
      |> put_flash(:error, "You must be logged in to create an advertisement.")
      |> redirect(to: ~p"/login")
    end
  end





  # Show form for editing an advertisement
  def edit(conn, %{"id" => id}) do
    try do
      advertisement = Advertisement.get_advertisement!(id)

      # Ensure the current user is the owner of the advertisement
      if advertisement.user_id != conn.assigns.current_user.id do
        conn
        |> put_flash(:error, "You cannot edit someone else's advertisement.")
        |> redirect(to: ~p"/advertisements")
      else
        changeset = Advertisement.changeset(advertisement, %{})
        render(conn, "edit.html", advertisement: advertisement, changeset: changeset)
      end
    rescue
      Ecto.NoResultsError ->
        conn
        |> put_flash(:error, "Advertisement not found.")
        |> redirect(to: ~p"/advertisements")
    end
  end




  # Update an advertisement
  def update(conn, %{"id" => id, "advertisement" => advertisement_params}) do
    advertisement = Advertisement.get_advertisement!(id)

    # Process new pictures only if they are provided
    new_pictures = process_pictures(advertisement_params["pictures"])

    # Retain old pictures if no new pictures are provided
    pictures = if new_pictures == [], do: advertisement.pictures, else: new_pictures

    # Merge pictures into the advertisement params
    updated_params = Map.put(advertisement_params, "pictures", pictures)

    case Advertisement.update_advertisement(advertisement, updated_params) do
      {:ok, _advertisement} ->
        conn
        |> put_flash(:info, "Advertisement updated successfully.")
        |> redirect(to: ~p"/advertisements")

      {:error, changeset} ->
        IO.inspect(changeset.errors, label: "Changeset Errors")  # Log errors
        render(conn, "edit.html", advertisement: advertisement, changeset: changeset)
    end
  end


    # Delete an advertisement
    def delete(conn, %{"id" => id}) do
      advertisement = Advertisement.get_advertisement!(id)
      {:ok, _advertisement} = Advertisement.delete_advertisement(advertisement)
      conn
      |> put_flash(:info, "Advertisement deleted successfully.")
      |> redirect(to: ~p"/advertisements")
    end

  defp process_pictures(nil), do: []
  defp process_pictures(pictures) when is_list(pictures) do
    Enum.map(pictures, fn
      %Plug.Upload{filename: filename, path: temp_path} ->
        save_picture(filename, temp_path)

      _ ->
        nil
    end)
    |> Enum.reject(&is_nil/1) # Remove any nil values
  end

  defp save_picture(filename, temp_path) do
    target_path = Path.join(["priv/static/adverts", filename])

    case File.cp(temp_path, target_path) do
      :ok ->
        "/adverts/#{filename}"  # Return the relative path to store in the DB

      {:error, reason} ->
        IO.puts("Failed to save picture: #{reason}")
        nil
    end
  end



# Show advertisement details
  def show(conn, %{"id" => id}) do
    advertisement = Advertisement.get_advertisement!(id)
    user_id = advertisement.user_id
    all_advertisements = Repo.all(Advertisement)
    recommended_properties = Advertisement.get_similar_advertisements(advertisement, all_advertisements)

    current_user = if user_id, do: Repo.get(Proptracker.Accounts.User, advertisement.user_id), else: nil
    render(conn, "show.html",
    advertisement: advertisement,
    recommended_properties: recommended_properties,
    current_user: current_user
    )
  end




  # Change the status of an advertisement
  def change_status(conn, %{"id" => id, "current_state" => current_state}) do
    advertisement = Advertisement.get_advertisement!(id)

    # Determine the new state based on the current state
    new_state = next_state(current_state)

    case Advertisement.update_advertisement(advertisement, %{state: new_state}) do
      {:ok, _advertisement} ->
        # Check if the request is AJAX (XHR)
        if get_req_header(conn, "x-requested-with") == ["XMLHttpRequest"] do
          # If it's an AJAX request, respond with JSON
          json(conn, %{status: "success", new_state: new_state})
        else
          # If it's a regular request, use the flash message and redirect
          conn
          |> put_flash(:info, "Advertisement status updated successfully.")
          |> redirect(to: ~p"/advertisements/#{id}")
        end

      {:error, _changeset} ->
        # Handle error and respond with the flash message
        conn
        |> put_flash(:error, "Failed to update advertisement status.")
        |> redirect(to: ~p"/advertisements/#{id}")
    end

  end
  defp next_state("available"), do: "reserved"
  defp next_state("reserved"), do: "sold/rented"
  defp next_state("sold/rented"), do: "available"


  def recommend_price(conn, %{"advertisement_id" => advertisement_id, "square_meters" => square_meters, "location" => location, "rooms" => rooms}) do
    # Ensure square_meters is an integer
    square_meters =
      cond do
        is_integer(square_meters) -> square_meters
        is_binary(square_meters) ->
          case Integer.parse(square_meters) do
            {value, _} -> value
            :error -> 0
          end
        true -> 0
      end

    # Ensure rooms is an integer
    rooms =
      cond do
        is_integer(rooms) -> rooms
        is_binary(rooms) ->
          case Integer.parse(rooms) do
            {value, _} -> value
            :error -> 0
          end
        true -> 0
      end

    # Ensure location is a string
    location = if is_binary(location), do: location, else: ""

    # Fetch recommended price if inputs are valid
    recommended_price =
      if square_meters > 0 and rooms > 0 and location != "" do
        Advertisement.get_recommended_price(advertisement_id, square_meters, rooms, location)
        |> case do
          nil -> 0
          price -> price
          |> Decimal.to_float() # Convert Decimal to Float
          |> Float.round(2) # Round to 2 decimal places

        end
      else
        0
      end

    json(conn, %{recommended_price: recommended_price})
  end



  defp validate_required_inputs(%{"square_meters" => "", "location" => "", "rooms" => ""}) do
    {:error, "All fields (rooms, location, square meters) must be filled."}
  end

  defp validate_required_inputs(%{"square_meters" => nil, "location" => nil, "rooms" => nil}) do
    {:error, "All fields (rooms, location, square meters) must be filled."}
  end

  defp validate_required_inputs(_params), do: :ok

  #change property interest:
  def change_interest(conn, %{"id" => id, "current_interest" => current_interest}) do

    advertisement = Advertisement.get_advertisement!(id)

    new_interest = next_interest(current_interest)

    # Using the new function to update the interest status only
    case Advertisement.update_interest(advertisement, new_interest) do
      {:ok, updated_advertisement} ->  # Handle the successful update case

        if get_req_header(conn, "x-requested-with") == ["XMLHttpRequest"] do
          json(conn, %{status: "success", new_interest: new_interest})
        else
          conn
          |> put_flash(:info, "Advertisement interest updated successfully.")
          |> redirect(to: ~p"/advertisements/#{id}")
        end

      {:error, changeset} ->  # Handle the failure case
        Logger.error("Failed to update advertisement interest: #{inspect(changeset)}")

        conn
        |> put_flash(:error, "Failed to update advertisement interest.")
        |> redirect(to: ~p"/advertisements/#{id}")
    end
  end

  defp next_interest("interest"), do: "not interest"
  defp next_interest("not interest"), do: "interest"

  #favorite ads of users
  def toggle_favorite(conn, %{"id" => id}) do
    current_user_id = get_session(conn, :user_id)

    # Ensure the current user is logged in
    if current_user_id == nil do
      conn
      |> put_status(:unauthorized)
      |> json(%{status: "error", message: "User is not logged in"})
      |> halt()
    else
      # Fetch the current user from the database
      current_user = Repo.get(User, current_user_id)

      if current_user == nil do
        conn
        |> put_status(:unauthorized)
        |> json(%{status: "error", message: "User not found"})
        |> halt()
      else
        # Fetch the advertisement
        advertisement = Advertisement.get_advertisement!(id)

        # Check if the advertisement is already in the favorite list
        is_favorite = Enum.member?(current_user.favorite_advertisements, advertisement.id)

        # Toggle the favorite status
        updated_favorites =
          if is_favorite do
            # Remove the advertisement ID from the list
            List.delete(current_user.favorite_advertisements, advertisement.id)
          else
            # Add the advertisement ID to the list
            [advertisement.id | current_user.favorite_advertisements]
          end

        # Update the user record with the new favorite list
        changeset = Ecto.Changeset.change(current_user, %{favorite_advertisements: updated_favorites})

        case Repo.update(changeset) do
          {:ok, _user} ->
            json(conn, %{status: "success", is_favorite: !is_favorite})

          {:error, changeset} ->
            IO.inspect(changeset.errors, label: "Changeset errors")
            conn
            |> put_status(:unprocessable_entity)
            |> json(%{status: "error", message: "Failed to update favorites"})
        end
      end
    end
  end


  # Display the current user's favorite advertisements that are 'available'
  def my_favorites(conn, _params) do
    current_user_id = get_session(conn, :user_id)

    # Ensure the current user is logged in
    if current_user_id == nil do
      conn
      |> put_status(:unauthorized)
      |> redirect(to: "/")
    else
      # Fetch the current user from the database
      current_user = Repo.get(User, current_user_id)

      # Check if user was found
      if current_user == nil do
        conn
        |> put_status(:unauthorized)
        |> json(%{status: "error", message: "User not found"})
        |> halt()
      else
        # Get the favorite advertisement IDs from the user's record
        favorite_ad_ids =
          case current_user.favorite_advertisements do
            nil -> []  # If it's nil, default to an empty list
            ads when is_list(ads) -> ads  # If it's already a list, use it
            ads when is_binary(ads) ->
              # If the data is a string, try to parse it as a list
              try do
                Jason.decode!(ads)
              rescue
                _ -> []  # If parsing fails, default to an empty list
              end
            _ -> []  # Default to an empty list for any other cases
          end

        IO.inspect(favorite_ad_ids, label: "Favorite Advertisement IDs after parsing")

        # Fetch the advertisements that are in the user's favorites list and have 'available' state
        advertisements =
          Advertisement
          |> where([a], a.id in ^favorite_ad_ids and a.state == "available")
          |> Repo.all()

        IO.inspect(advertisements, label: "Fetched Advertisements")

        # Render the favorites.html template and pass the advertisements
        render(conn, "favorites.html", advertisements: advertisements, current_user: current_user)
      end
    end
  end

  #toggle interest for ads:
  def toggle_interest(conn, %{"id" => id}) do
    current_user_id = get_session(conn, :user_id)

    # Ensure the current user is logged in
    if current_user_id == nil do
      conn
      |> put_status(:unauthorized)
      |> json(%{status: "error", message: "User is not logged in"})
      |> halt()
    else
      # Fetch the current user from the database
      current_user = Repo.get(User, current_user_id)

      if current_user == nil do
        conn
        |> put_status(:unauthorized)
        |> json(%{status: "error", message: "User not found"})
        |> halt()
      else
        # Fetch the advertisement
        advertisement = Advertisement.get_advertisementInterest!(id)

        # Check if the advertisement is in the interested list
        is_interested = Enum.member?(current_user.interested, advertisement.id)

        updated_interested =
          if is_interested do
            # Remove from interested and ensure not in not_interested
            List.delete(current_user.interested, advertisement.id)
          else
            # Add to interested and ensure it's removed from not_interested
            [advertisement.id | current_user.interested]
          end

        updated_not_interested =
          if is_interested do
            current_user.not_interested
          else
            List.delete(current_user.not_interested, advertisement.id)
          end

        # Update the user record with the new interest lists
        changeset =
          Ecto.Changeset.change(current_user, %{
            interested: updated_interested,
            not_interested: updated_not_interested
          })

        case Repo.update(changeset) do
          {:ok, _user} ->
            json(conn, %{status: "success", is_interested: !is_interested})

          {:error, changeset} ->
            IO.inspect(changeset.errors, label: "Changeset errors")
            conn
            |> put_status(:unprocessable_entity)
            |> json(%{status: "error", message: "Failed to update interest"})
        end
      end
    end
  end


end
