defmodule ProptrackerWeb.SearchController do
  use ProptrackerWeb, :controller

  alias Proptracker.Accounts.Advertisement
  alias Proptracker.Repo
  import Ecto.Query

  # Action to handle search by type
  def prop_type(conn, %{
    "submitted" => "true",
    "type" => type,
    "location" => location,
    "query" => query,
    "state" => state,
    "min_price" => min_price,
    "max_price" => max_price,
    "min_rooms" => min_rooms,
    "max_rooms" => max_rooms
    }) do
    user_id = get_session(conn, :user_id)

    # Retrieve the current user if logged in
    current_user = if user_id, do: Repo.get(Proptracker.Accounts.User, user_id), else: nil

    query = query || ""  # Default to empty string if query is nil
    type = type || ""
    location = location || ""
    state = state || ""
    min_price = if min_price == "", do: nil, else: String.to_integer(min_price)
    max_price = if max_price == "", do: nil, else: String.to_integer(max_price)
    min_rooms = if min_rooms == "", do: nil, else: String.to_integer(min_rooms)
    max_rooms = if max_rooms == "", do: nil, else: String.to_integer(max_rooms)

    # Validation for max_price being greater than or equal to min_price
    if min_price && max_price && max_price < min_price do
      conn
      |> put_flash(:error, "Max price cannot be less than min price.")
      |> redirect(to: "/")
      |> halt()
    else
      # Additional validation for max_rooms >= min_rooms
      if min_rooms && max_rooms && max_rooms < min_rooms do
        conn
        |> put_flash(:error, "Max rooms cannot be less than min rooms.")
        |> redirect(to: "/")
        |> halt()
      else
        # Save search only if there's an actual query
        if current_user != nil and (type != "" or location != "" or query != "") do
          changeset =
            Proptracker.Accounts.SearchHistory.changeset(%Proptracker.Accounts.SearchHistory{}, %{
              type: type,
              location: location,
              query: query,
              user_id: current_user.id
            })

          case Repo.insert(changeset) do
            {:ok, _} -> :ok
            {:error, changeset} -> IO.inspect(changeset, label: "Failed to save search history")
          end
        end

        # Build the query dynamically based on the parameters
        advertisements =
          Advertisement
          |> (fn q -> if type != "", do: from(a in q, where: a.type == ^type), else: q end).()
          |> (fn q -> if query != "", do: from(a in q, where: ilike(a.title, ^"%#{query}%")), else: q end).()
          |> (fn q -> if location != "", do: from(a in q, where: ilike(a.location, ^"%#{location}%")), else: q end).()
          |> (fn q -> if state != "", do: from(a in q, where: a.state == ^state), else: q end).()
          |> (fn q -> if min_price != nil, do: from(a in q, where: a.price >= ^min_price), else: q end).()
          |> (fn q -> if max_price != nil, do: from(a in q, where: a.price <= ^max_price), else: q end).()
          |> (fn q -> if min_rooms != nil, do: from(a in q, where: a.rooms >= ^min_rooms), else: q end).()
          |> (fn q -> if max_rooms != nil, do: from(a in q, where: a.rooms <= ^max_rooms), else: q end).()
          |> where([a], a.state != "sold/rented")
          |> order_by([a], asc: a.state)  # Add ordering by 'state', available first
          |> Repo.all()

        # Pass the parameters and results to the template
        render(conn, ProptrackerWeb.PageHTML, "home1.html",
          advertisements: advertisements,
          current_user: current_user,
          type: type,
          state: state,
          location: location,
          query: query,
          min_price: min_price,
          max_price: max_price,
          min_rooms: min_rooms,
          max_rooms: max_rooms,
          layout: false
        )
      end
    end
  end

  # Action to handle past searches
  def past_searches(conn, _params) do
    user_id = get_session(conn, :user_id)

    if user_id do
      search_history =
        Proptracker.Accounts.SearchHistory
        |> where([s], s.user_id == ^user_id)
        |> order_by([s], desc: s.inserted_at)
        |> Repo.all()

      render(conn, ProptrackerWeb.PageHTML, "past_searches.html",
        search_history: search_history,
        layout: false
      )
    else
      conn
      |> put_flash(:error, "You need to log in to access past searches.")
      |> redirect(to: "/login")
    end
  end
end
