defmodule ProptrackerWeb.UserController do
  use ProptrackerWeb, :controller

  alias Hex.API.User
  alias Proptracker.Repo
  alias Proptracker.Accounts.User
  alias Proptracker.Accounts.Advertisement
  import Ecto.Query

  def index(conn, _params) do
    # Get the current authenticated user from the connection assigns
    current_user = conn.assigns[:current_user]

    if current_user do
      # Fetch only the current user's data
      user = Repo.get!(User, current_user.id)
      render(conn, "index.html", users: [user]) # Pass as a list for consistency
    else
      # Redirect or render an error page if no user is logged in
      conn
      |> put_flash(:error, "You need to log in to view this page.")
      |> redirect(to: ~p"/login")
      |> halt()
    end
  end

  def new(conn, _params) do
    changeset = User.changeset(%User{}, %{})
    render(conn, "new.html", changeset: changeset)
  end

  def create(conn, %{"user" => user_params}) do
    changeset = User.changeset(%User{}, user_params)
    case Repo.insert(changeset) do
      {:ok, _user} ->
        conn
          |> put_flash(:info, "User created successfully.")
          |> redirect(to: ~p"/users")
      {:error, %Ecto.Changeset{} = changeset} ->
        render(conn, "show.html", changeset: changeset)
    end
  end

  def edit(conn, %{"id" => id}) do
    current_user = conn.assigns.current_user

    if current_user && current_user.id == String.to_integer(id) do
      user = Repo.get!(User, id)
      changeset = User.changeset(user, %{})
      render(conn, "edit.html", user: user, changeset: changeset)
    else
      conn
      |> put_flash(:error, "You are not authorized to perform this activity.")
      |> render("edit.html", user: current_user, changeset: User.changeset(current_user, %{}))
    end
  end

  def update(conn, %{"id" => id, "user" => user_params}) do
    user = Repo.get!(User, id)
    changeset = User.changeset(user, user_params)

    case Repo.update(changeset) do
      {:ok, _user} ->
        conn
        |> put_flash(:info, "User updated successfully.")
        |> redirect(to: ~p"/users/#{user.id}")

      {:error, %Ecto.Changeset{} = changeset} ->
        conn
        |> put_flash(:error, "Failed to update user. Please check the errors below.")
        |> render("edit.html", changeset: changeset, user: user)
    end
  end

  def show(conn, %{"id" => id}) do
    current_user = conn.assigns.current_user

    if current_user && current_user.id == String.to_integer(id) do
      user = Repo.get!(User, id)
      render(conn, "show.html", user: user)
    else
      conn
      |> put_flash(:error, "Don't even think about such an act :@.")
      |> render("show.html", user: current_user)
    end
  end


  def delete(conn, %{"id" => id}) do
    user = Repo.get(User, id)

    # Check if the current user is allowed to delete the user
    if conn.assigns[:current_user].id == user.id or conn.assigns[:current_user].role == "admin" do
      Repo.delete!(user)
      conn
      |> put_flash(:info, "User deleted successfully.")
      |> redirect(to: ~p"/login")
    else
      # If not authorized, redirect to users page with error message
      conn
      |> put_flash(:error, "You do not have permission to delete this user.")
      |> redirect(to: ~p"/users")  # Redirect to users page instead of login
    end
  end


  def owner_profile(conn, %{"id" => advertisement_id}) do
    # Fetch the advertisement by ID
    advertisement = Advertisement.get_advertisement!(advertisement_id)

    # Ensure the advertisement has a valid user_id
    if advertisement.user_id do
      # Fetch the owner profile only if user_id is valid
      owner = Repo.get!(User, advertisement.user_id)

      # Fetch all advertisements by the same owner
      owner_ads = Advertisement.list_advertisements_by_user(advertisement.user_id)

      # Render the owner's profile along with their advertisements
      render(conn, "owner_profile.html", user: owner, advertisements: owner_ads)
    else
      # Handle the case where the user_id is nil (e.g., render an error page)
      conn
      |> put_flash(:error, "Owner information not available")
      |> redirect(to: "/")
    end
  end


end
