defmodule ProptrackerWeb.ShowFavoritesTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01], phone_number: "123456789"}
  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }

  defp create_user(_) do
    {:ok, user} = Repo.insert(User.changeset(%User{}, @valid_user_attrs))
    %{user: user}
  end

  defp create_advertisement(%{user: user}) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, advertisement_attrs))
    %{advertisement: advertisement}
  end

  defp login_user(conn, user) do
    conn
    |> post(~p"/login", %{"user" => %{"username" => user.username, "password" => user.password}})
    |> recycle() # Recycle the connection to persist session data
  end

  describe "GET /my_favorites" do
    setup [:create_user, :create_advertisement]

    test "displays the user's favorite advertisements when logged in", %{conn: conn, user: user, advertisement: advertisement} do
      # Add the advertisement to the user's favorites
      conn = login_user(conn, user)
      conn = post(conn, ~p"/advertisements/#{advertisement.id}/toggle_favorite", %{is_favorite: true})

      # Now, visit the "My Favorites" page
      conn = get(conn, "/my_favorites")

      # Check if the page contains the expected advertisement
      assert html_response(conn, 200) =~ advertisement.title
      assert html_response(conn, 200) =~ advertisement.price |> Decimal.to_string()
      assert html_response(conn, 200) =~ advertisement.location
    end

    test "redirects non-logged-in users to the homepage when trying to access my favorites", %{conn: conn} do
      # Access "My Favorites" without logging in
      conn = get(conn, "/my_favorites")

      # Assert that the response is a 401 (Unauthorized)
      assert html_response(conn, 401)

      # Optionally, check if the flash message indicates the user must be logged in
      assert get_flash(conn, :error) == nil
    end
  end
end
