defmodule Proptracker.UserTest do
  use ProptrackerWeb.ConnCase, async: true
  alias Proptracker.Repo
  alias Proptracker.Accounts.User
  alias ProptrackerWeb.UserController
  alias ProptrackerWeb.SessionController
  alias ProptrackerWeb.Router.Helpers, as: Routes

  @valid_attrs %{
    name: "John Doe",
    username: "johndoe",
    date_of_birth: ~D[1990-01-01],
    password: "securepassword",
    user_details: "Sample user details",
    phone_number: "1234567890"
  }

  @updated_attrs %{
    name: "Updated Name",
    username: "updatedusername",
    date_of_birth: ~D[1991-02-02],
    password: "newsecurepassword",
    user_details: "Updated user details",
    phone_number: "1122334455"
  }

  @invalid_attrs %{
    name: "",
    username: "",
    date_of_birth: nil,
    password: "",
    user_details: "",
    phone_number: ""
  }

  # Setup block: creating a user and logging them in for tests that require authentication
  setup %{conn: conn} do
    unique_username = "johndoe_#{System.unique_integer()}"

    # Register a new user using the create_user action of SessionController (simulate registration)
    conn = post(conn, "/register", %{
      "user" => %{
        "username" => unique_username,
        "password" => "securepassword",
        "name" => "John Doe",
        "date_of_birth" => "1990-01-01",
        "phone_number" => "1234567890",
        "user_details" => "Test user"
      }
    })

    # Try to fetch the user from the database
    user = Repo.get_by(Proptracker.Accounts.User, username: unique_username)

    # Log the user in (simulate the login action)
    conn = post(conn, "/login", %{
      "user" => %{
        "username" => unique_username,
        "password" => "securepassword"
      }
    })

    {:ok, conn: conn, user: user}
  end

  describe "user creation" do
    test "creates user with valid attributes", %{conn: conn} do
      # Test valid user creation
      conn = post(conn, "/register", user: @valid_attrs)
      assert redirected_to(conn) == ~p"/login"
      assert get_flash(conn, :info) == "Account created successfully."

      # Verify that the user was created in the database
      user = Repo.get_by(User, username: "johndoe")
      assert user
      assert user.name == "John Doe"
      assert user.username == "johndoe"
    end

    test "does not create user with invalid attributes", %{conn: conn} do
      # Test invalid user creation
      conn = post(conn, "/register", user: @invalid_attrs)
      assert html_response(conn, 200) =~ "Error creating account."
    end
  end

  describe "user update" do
    test "successfully updates the current user's details", %{conn: conn, user: user} do
      conn = put(conn, ~p"/users/#{user.id}", user: @updated_attrs)

      # Check if the user was redirected and if flash message appears
      assert redirected_to(conn) == ~p"/users/#{user.id}"
      assert get_flash(conn, :info) == "User updated successfully."

      # Verify the user is updated in the database
      updated_user = Repo.get(User, user.id)
      assert updated_user.name == "Updated Name"
      assert updated_user.username == "updatedusername"
      assert updated_user.date_of_birth == ~D[1991-02-02]
      assert updated_user.user_details == "Updated user details"
      assert updated_user.phone_number == "1122334455"
      assert updated_user.password == "newsecurepassword"

    end

    test "fails to update user with invalid data", %{conn: conn, user: user} do
      conn = put(conn, ~p"/users/#{user.id}", user: @invalid_attrs)

      # Check if it renders the edit form again with error flash message
      assert html_response(conn, 200) =~ "Failed to update user"
      assert get_flash(conn, :error) == "Failed to update user. Please check the errors below."
    end
  end

  describe "user retrieval" do
    test "retrieves and displays user data on /users/:id route", %{conn: conn, user: user} do
      # Request the user data via the show route (assuming the user route is /users/:id)
      conn = get(conn, ~p"/users/#{user.id}")

      # Verify the response contains the user data
      assert html_response(conn, 200) =~ "John Doe"
      assert html_response(conn, 200) =~ "johndoe"
      assert html_response(conn, 200) =~ "1990-01-01"
      assert html_response(conn, 200) =~ "securepassword"
    end
  end
end
