defmodule ProptrackerWeb.AdvertisementShowTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01], phone_number: "123456789"}
  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }

  # Create user helper function
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Create advertisement helper function, preload user association
  defp create_advertisement(%{user: user}) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    changeset = Advertisement.changeset(%Advertisement{}, advertisement_attrs)
    {:ok, advertisement} = Repo.insert(changeset)
    advertisement = Repo.preload(advertisement, :user)
    %{advertisement: advertisement}
  end

  # Login helper function
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "show advertisement" do
    setup [:create_user, :create_advertisement]

    test "successfully renders the advertisement details", %{conn: conn, advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure user is logged in (session contains user_id)
      assert get_session(conn, :user_id) == advertisement.user.id

      # Simulate a GET request to the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Check if the advertisement's details are correctly rendered
      assert html_response(conn, 200) =~ "Sample Ad"
      assert html_response(conn, 200) =~ "Sample Description"
      assert html_response(conn, 200) =~ "Sample City"
      assert html_response(conn, 200) =~ "100"
      assert html_response(conn, 200) =~ "2"
      assert html_response(conn, 200) =~ "available"
    end

    test "does not show advertisement details to a different user", %{conn: conn, advertisement: advertisement} do
      # Create a second user
      {:ok, other_user} = Repo.insert(%User{} |> User.changeset(%{name: "Jane Doe", username: "jane_doe", password: "password123", date_of_birth: ~D[1992-05-01], phone_number: "987654321"}))

      # Simulate logging in with the second user
      conn = login_user(conn, other_user)

      # Ensure the second user is logged in (session contains user_id)
      assert get_session(conn, :user_id) == other_user.id

      # Simulate a GET request to the advertisement's show page (should fail since this is not the correct owner)
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")
    end
    test "advertisement is successfully saved to the database", %{conn: conn, user: user} do
      # Simulate logging in the user
      conn = login_user(conn, user)

      # Ensure user is logged in (session contains user_id)
      assert get_session(conn, :user_id) == user.id

      # Simulate a POST request to create an advertisement
      conn =
        post(conn, ~p"/advertisements", %{
          "advertisement" => Map.put(@valid_advertisement_attrs, :user_id, user.id)
        })

      # Retrieve the advertisement from the database
      saved_advertisement = Repo.get_by(Advertisement, title: "Sample Ad")

      # Check if the advertisement exists in the database
      assert saved_advertisement != nil
      assert saved_advertisement.title == "Sample Ad"
      assert saved_advertisement.description == "Sample Description"
      assert saved_advertisement.location == "Sample City"
      assert saved_advertisement.user_id == user.id
      assert saved_advertisement.state == "available"
    end
  end
end
