defmodule PropTrackerWeb.OwnerProfileTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  # Valid advertisement attributes
  @valid_advertisement_attrs %{
    title: "Test Advertisement",
    description: "A great property",
    price: Decimal.new("1500.00"),
    square_meters: 80,
    location: "Sample Location",
    rooms: 3,
    floor: 2,
    total_floors: 5,
    type: "sold/rented"
  }

  # Valid user attributes
  @valid_user_attrs %{
    name: "Test Owner",
    username: "testowner",
    password: "password",
    date_of_birth: ~D[1990-01-01],  # Example date
    phone_number: "1234567890"    # Example phone number
  }

  # Private function to create a user
  defp create_user(attrs \\ @valid_user_attrs) do
    changeset = User.changeset(%User{}, attrs)

    case Repo.insert(changeset) do
      {:ok, user} -> user
      {:error, changeset} ->
        IO.inspect(changeset.errors, label: "User creation error")
        raise "Failed to create user"
    end
  end

  # Private function to create an advertisement for a user
  defp create_advertisement(user) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, advertisement_attrs))
    advertisement
  end

  describe "GET /owner_profile" do
    test "renders owner's profile and their advertisements when advertisement exists", %{conn: conn} do
      # Setup: Create an owner and an advertisement
      owner = create_user()  # This uses the default valid user attributes
      advertisement = create_advertisement(owner)

      # Act: Make a request to the owner_profile action using the direct path
      conn = get(conn, "/advertisements/#{advertisement.id}/owner")

      # Assert: Check that the owner's profile is rendered correctly
      assert html_response(conn, 200) =~ "Owner's Profile"
      assert html_response(conn, 200) =~ owner.name  # Check for owner's name
      assert html_response(conn, 200) =~ advertisement.title
    end

    test "redirects when advertisement has no user_id", %{conn: conn} do
      # Setup: Create an advertisement with nil user_id
      advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, nil)
      advertisement_changeset = Advertisement.changeset(%Advertisement{}, advertisement_attrs)
      {:ok, advertisement} = Repo.insert(advertisement_changeset)

      # Act: Make a request to the owner_profile action using the direct path
      conn = get(conn, "/advertisements/#{advertisement.id}/owner")

      # Assert: Check that the user is redirected with an error message
      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) == "Owner information not available"
    end

    test "raises an error when advertisement does not exist", %{conn: conn} do
      # Act: Make a request to the owner_profile action with a non-existent advertisement ID using the direct path
      assert_raise Ecto.NoResultsError, fn ->
        get(conn, "/advertisements/-1/owner")
      end
    end
  end
end
