defmodule ProptrackerWeb.SimilarAdvertisementTest do
  use Proptracker.DataCase, async: true

  alias Proptracker.Accounts.Advertisement

  describe "calculate_similarity/2" do
    setup do
      advertisement = %Advertisement{
        id: 1,
        title: "Cozy Apartment",
        price: Decimal.new(100_000),
        square_meters: 50,
        rooms: 3,
        location: "Downtown"
      }

      similar_advertisement = %Advertisement{
        id: 2,
        title: "Modern Apartment",
        price: Decimal.new(101_000),
        square_meters: 49.5,
        rooms: 3, # Same number of rooms
        location: "Downtown"
      }

      moderately_similar_advertisement = %Advertisement{
        id: 3,
        title: "Spacious Apartment",
        price: Decimal.new(102_000),
        square_meters: 49.3,
        rooms: 2, # ±1 room difference
        location: "Suburb"
      }

      dissimilar_advertisement = %Advertisement{
        id: 4,
        title: "Luxury Villa",
        price: Decimal.new(150_000), # ±50% difference
        square_meters: 70, # ±40% difference
        rooms: 5, # ±2 rooms difference
        location: "Uptown"
      }

      [
        advertisement: advertisement,
        similar_advertisement: similar_advertisement,
        moderately_similar_advertisement: moderately_similar_advertisement,
        dissimilar_advertisement: dissimilar_advertisement
      ]
    end

    test "calculates similarity for highly similar advertisements", context do
      {score, factors} =
        Advertisement.calculate_similarity(
          context[:advertisement],
          context[:similar_advertisement]
        )

      IO.inspect({factors.price_similarity, factors.surface_similarity, factors.rooms_similarity, score}, label: "Highly Similarity Breakdown")

      assert score >= 90.0
      assert factors.price_similarity > 90.0
      assert factors.surface_similarity > 90.0
      assert factors.rooms_similarity == 100
    end

    test "calculates similarity for moderately similar advertisements", context do
      {score, factors} =
        Advertisement.calculate_similarity(
          context[:advertisement],
          context[:moderately_similar_advertisement]
        )
      IO.inspect({factors.price_similarity, factors.surface_similarity, factors.rooms_similarity, score}, label: "Moderately Similarity Breakdown")

      assert score >= 70.0
      assert factors.price_similarity > 80.0
      assert factors.surface_similarity >= 90.0
      assert factors.rooms_similarity == 70
    end

    test "calculates similarity for dissimilar advertisements", context do
      {score, factors} =
        Advertisement.calculate_similarity(
          context[:advertisement],
          context[:dissimilar_advertisement]
        )

      assert score < 50.0
      assert factors.price_similarity < 50.0
      assert factors.surface_similarity < 50.0
      assert factors.rooms_similarity == 0
    end
  end

  describe "get_similar_advertisements/2" do
    setup do
      advertisement = %Advertisement{
        id: 1,
        title: "Cozy Apartment",
        price: Decimal.new(100_000),
        square_meters: 50,
        rooms: 3,
        location: "Downtown"
      }

      ads = [
        %Advertisement{
          id: 2,
          title: "Modern Apartment",
          price: Decimal.new(105_000),
          square_meters: 48,
          rooms: 3,
          location: "Downtown"
        },
        %Advertisement{
          id: 3,
          title: "Spacious Apartment",
          price: Decimal.new(120_000),
          square_meters: 55,
          rooms: 2,
          location: "Suburb"
        },
        %Advertisement{
          id: 4,
          title: "Luxury Villa",
          price: Decimal.new(150_000),
          square_meters: 70,
          rooms: 5,
          location: "Uptown"
        },
        %Advertisement{
          id: 5,
          title: "Affordable Apartment",
          price: Decimal.new(95_000),
          square_meters: 49,
          rooms: 3,
          location: "Downtown"
        }
      ]

      [advertisement: advertisement, ads: ads]
    end

    test "returns a list of similar advertisements sorted by similarity", context do
      similar_ads =
        Advertisement.get_similar_advertisements(context[:advertisement], context[:ads])

      assert length(similar_ads) == 2
      assert Enum.at(similar_ads, 0).similarity > Enum.at(similar_ads, 1).similarity

      assert Enum.all?(similar_ads, fn ad -> ad.similarity >= 70 end)
    end

    test "returns an empty list when no similar advertisements are found", context do
      ads = [
        %Advertisement{
          id: 6,
          title: "Faraway Mansion",
          price: Decimal.new(200_000),
          square_meters: 100,
          rooms: 6,
          location: "Remote Area"
        }
      ]

      similar_ads =
        Advertisement.get_similar_advertisements(context[:advertisement], ads)

      assert similar_ads == []
    end

    test "handles edge cases where all advertisements are excluded", context do
      similar_ads =
        Advertisement.get_similar_advertisements(context[:advertisement], [context[:advertisement]])

      assert similar_ads == []
    end
  end
end
