defmodule ProptrackerWeb.SearchControllerTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  defp create_user(_) do
    {:ok, user} =
      %User{}
      |> User.changeset(%{
        name: "Test User",
        username: "test_user",
        password: "password123",
        date_of_birth: ~D[1990-01-01],
        phone_number: "123456789"
      })
      |> Repo.insert()

    %{user: user}
  end

  defp create_advertisements(%{user: user}) do
    valid_advertisement_attrs = %{
      type: "rent",
      title: "Beautiful Apartment",
      location: "Tallinn",
      reference: "REF-#{:crypto.strong_rand_bytes(4) |> Base.encode16()}",
      price: 1500,
      square_meters: 75,
      rooms: 3,
      state: "available",
      user_id: user.id
    }

    {:ok, advertisement} =
      %Advertisement{}
      |> Advertisement.changeset(valid_advertisement_attrs)
      |> Repo.insert()

    %{advertisement: advertisement}
  end

  describe "property search" do
    setup [:create_user, :create_advertisements]

    test "returns advertisements matching the search query", %{conn: conn, advertisement: advertisement} do
      conn =
        get(conn, "/prop_type", %{
          "type" => "rent",
          "location" => "Tallinn",
          "query" => "Apartment",
          "state" => "available",
          "submitted" => "true",
          "min_price" => "1000",
          "max_price" => "2000",
          "min_rooms" => "2",
          "max_rooms" => "4"
        })

      assert html_response(conn, 200) =~ advertisement.title
    end

    test "returns all advertisements if no filters are applied", %{conn: conn, advertisement: advertisement} do
      conn =
        get(conn, "/prop_type", %{
          "submitted" => "true",
          "type" => "rent",
          "location" => "Tallinn",
          "query" => "",
          "state" => "",
          "min_price" => "",
          "max_price" => "",
          "min_rooms" => "",
          "max_rooms" => ""
        })

      assert html_response(conn, 200) =~ advertisement.title
    end

    test "returns no results if the search criteria do not match any advertisements", %{conn: conn} do
      conn =
        get(conn, "/prop_type", %{
          "submitted" => "true",
          "type" => "sell",
          "location" => "Unknown",
          "query" => "",
          "state" => "",
          "min_price" => "",
          "max_price" => "",
          "min_rooms" => "4",
          "max_rooms" => "5"
        })

      refute html_response(conn, 200) =~ "Beautiful Apartment"
    end

    test "returns no results if rooms criteria are not met", %{conn: conn} do
      conn =
        get(conn, "/prop_type", %{
          "submitted" => "true",
          "type" => "rent",
          "location" => "Tallinn",
          "query" => "",
          "state" => "",
          "min_price" => "1000",
          "max_price" => "2000",
          "min_rooms" => "4",
          "max_rooms" => "5"
        })

      refute html_response(conn, 200) =~ "Beautiful Apartment"
    end

    test "shows error when max_price is less than min_price", %{conn: conn} do
      conn =
        get(conn, "/prop_type", %{
          "submitted" => "true",
          "type" => "sell",
          "location" => "Unknown",
          "query" => "Apartment",
          "state" => "Available",
          "min_price" => "4000",
          "max_price" => "200",
          "min_rooms" => "1",
          "max_rooms" => "3"
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) == "Max price cannot be less than min price."
    end

    test "shows error when max_rooms is less than min_rooms", %{conn: conn} do
      conn =
        get(conn, "/prop_type", %{
          "submitted" => "true",
          "type" => "rent",
          "location" => "Tallinn",
          "query" => "",
          "state" => "",
          "min_price" => "1000",
          "max_price" => "2000",
          "min_rooms" => "5",
          "max_rooms" => "2"
        })

      assert redirected_to(conn) == "/"
      assert get_flash(conn, :error) == "Max rooms cannot be less than min rooms."
    end
  end
end
