defmodule ProptrackerWeb.AdvertisementInterestTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{
    name: "John Doe",
    username: "johndoe",
    date_of_birth: ~D[1990-01-01],
    password: "securepassword",
    user_details: "Sample user details",
    phone_number: "1234567890"
  }
  @other_user_attrs %{
    name: "Jane Doe",
    username: "janedoe",
    date_of_birth: ~D[1990-02-01],
    password: "anotherpassword",
    user_details: "Other user details",
    phone_number: "0987654321"
  }
  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }

  defp create_users(_) do
    unique_username = "johndoe_#{System.unique_integer()}"
    unique_other_username = "janedoe_#{System.unique_integer()}"

    {:ok, user} = Repo.insert(User.changeset(%User{}, Map.put(@valid_user_attrs, :username, unique_username)))
    {:ok, other_user} = Repo.insert(User.changeset(%User{}, Map.put(@other_user_attrs, :username, unique_other_username)))
    %{user: user, other_user: other_user}
  end

  defp create_advertisement(%{user: user}) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, advertisement_attrs))
    %{advertisement: advertisement}
  end

  defp login_user(conn, user) do
    conn
    |> post(~p"/login", %{"user" => %{"username" => user.username, "password" => user.password}})
    |> recycle() # Recycle the connection to persist session data
  end

  describe "interest functionality" do
    setup [:create_users, :create_advertisement]

    test "displays 'Mark Not Interested' button for logged-in user on homepage", %{conn: conn, user: user, advertisement: advertisement} do
      conn = login_user(conn, user)
      conn = get(conn, "/") # Assuming the homepage is at "/home"

      # Check if the 'Mark Not Interested' button is displayed
      assert html_response(conn, 200) =~ "Mark Not Interested"
    end

    test "successfully toggles interest status from homepage", %{conn: conn, user: user, advertisement: advertisement} do
      conn = login_user(conn, user)
      advertisement_id = advertisement.id

      # Ensure the user is initially not interested
      conn = get(conn, "/")
      assert html_response(conn, 200) =~ "Mark Not Interested"

      # Simulate the toggle interest request
      conn = post(conn, ~p"/advertisements/#{advertisement_id}/toggle_interest", %{is_interested: false})

      # Check the response for success and updated interest status
      assert json_response(conn, 200)["status"] == "success"
      assert json_response(conn, 200)["is_interested"] == true

      # Reload the user to check the updated interest list
      updated_user = Repo.get(User, user.id)
      assert advertisement_id in updated_user.interested
    end

    test "successfully toggles interest status back to not interested", %{conn: conn, user: user, advertisement: advertisement} do
      # Initially mark the user as interested
      conn = login_user(conn, user)
      advertisement_id = advertisement.id
      conn = post(conn, ~p"/advertisements/#{advertisement_id}/toggle_interest", %{is_interested: false})

      # Toggle back to not interested
      conn = post(conn, ~p"/advertisements/#{advertisement_id}/toggle_interest", %{is_interested: true})

      # Check the response for success and updated interest status
      assert json_response(conn, 200)["status"] == "success"
      assert json_response(conn, 200)["is_interested"] == false

      # Reload the user to check the updated interest list
      updated_user = Repo.get(User, user.id)
      refute advertisement_id in updated_user.interested
    end

    test "prevents unauthenticated users from toggling interest", %{conn: conn, advertisement: advertisement} do
      conn = post(conn, ~p"/advertisements/#{advertisement.id}/toggle_interest", %{is_interested: false})
      assert json_response(conn, 401)["status"] == "error"
    end
  end
end
