defmodule ProptrackerWeb.SoldRentedTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{
    name: "Owner Name",
    username: "owner_username",
    password: "password123",
    date_of_birth: ~D[1990-01-01],
    phone_number: "123456789"
  }

  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"  # This one is for the available ad
  }

  @sold_advertisement_attrs %{
    title: "Sold Ad",
    pictures: ["path/to/image3.jpg"],
    description: "Sold Advertisement",
    type: "rent",
    price: Decimal.new("1500.00"),
    square_meters: 80,
    location: "Another City",
    rooms: 3,
    floor: 2,
    total_floors: 5,
    state: "sold/rented"  # This one is for a sold ad
  }

  # Create user helper function
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Create advertisement helper function, preload user association
  defp create_advertisements(%{user: user}) do
    # Create an available advertisement
    available_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, available_ad} = Repo.insert(Advertisement.changeset(%Advertisement{}, available_attrs))

    # Create a sold advertisement
    sold_attrs = Map.put(@sold_advertisement_attrs, :user_id, user.id)
    {:ok, sold_ad} = Repo.insert(Advertisement.changeset(%Advertisement{}, sold_attrs))

    %{available_ad: available_ad, sold_ad: sold_ad, user: user}
  end

  # Login helper function
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "homepage advertisements" do
    setup [:create_user, :create_advertisements]

    test "only available advertisements show on the homepage", %{conn: conn, available_ad: available_ad, sold_ad: sold_ad, user: user} do
      # Simulate a GET request to the homepage
      conn = get(conn, "/")

      # Check the response
      response = html_response(conn, 200)

      # Parse the response with Floki to extract ad titles
      ad_titles = Floki.find(response, ".advertisement-box .advertisement-title")
                    |> Enum.map(&Floki.text(&1))  # Extract the text of each title

      # Ensure the available ad is in the response
      assert available_ad.title in ad_titles

      # Ensure the sold ad is NOT in the response (it shouldn't appear on the homepage)
      refute sold_ad.title in ad_titles
    end
    test "sold/rented advertisements do not show on the homepage", %{conn: conn, available_ad: available_ad, sold_ad: sold_ad, user: user} do
      # Simulate a GET request to the homepage
      conn = get(conn, "/")

      # Check the response
      response = html_response(conn, 200)

      # Parse the response with Floki to extract ad titles
      ad_titles = Floki.find(response, ".advertisement-box .advertisement-title")
                    |> Enum.map(&Floki.text(&1))  # Extract the text of each title

      # Ensure the available ad is in the response
      assert available_ad.title in ad_titles

      # Ensure the sold ad is NOT in the response (sold/rented ads shouldn't appear on the homepage)
      refute sold_ad.title in ad_titles
    end
  end

end
