defmodule ProptrackerWeb.OwnerPhoneTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01], phone_number: "123456789"}
  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }

  # Create user helper function
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Create advertisement helper function
  defp create_advertisement(%{user: user}) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    changeset = Advertisement.changeset(%Advertisement{}, advertisement_attrs)
    {:ok, advertisement} = Repo.insert(changeset)
    advertisement = Repo.preload(advertisement, :user)
    %{advertisement: advertisement}
  end

  # Login helper function
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "owner phone visibility" do
    setup [:create_user, :create_advertisement]

    test "renders contact button and phone number hidden on initial load", %{conn: conn, advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Initial GET request to the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Verify the "Contact" button exists and the phone number is hidden
      assert html_response(conn, 200) =~ "Contact"
      assert html_response(conn, 200) =~ "id=\"contact-info\""
    end

    test "does not render contact button for the advertisement owner", %{conn: conn, advertisement: advertisement} do
      # Simulate logging in as the user who owns the advertisement
      conn = login_user(conn, advertisement.user)

      # Ensure user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Initial GET request to the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      assert html_response(conn, 200) =~ "Contact"
    end
  end
end
