defmodule ProptrackerWeb.AdvertisementFavoriteTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{
    name: "John Doe",
    username: "johndoe",
    date_of_birth: ~D[1990-01-01],
    password: "securepassword",
    user_details: "Sample user details",
    phone_number: "1234567890"
  }
  @other_user_attrs %{
    name: "John Doe",
    username: "jonedoe",
    date_of_birth: ~D[1990-01-01],
    password: "securepassword",
    user_details: "Sample user details",
    phone_number: "1234567890"
  }
  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }

  defp create_users(_) do
    {:ok, user} = Repo.insert(User.changeset(%User{}, @valid_user_attrs))
    {:ok, other_user} = Repo.insert(User.changeset(%User{}, @other_user_attrs))
    %{user: user, other_user: other_user}
  end

  defp create_advertisement(%{user: user}) do
    advertisement_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, advertisement_attrs))
    %{advertisement: advertisement}
  end

  defp login_user(conn, user) do
    conn
    |> post(~p"/login", %{"user" => %{"username" => user.username, "password" => user.password}})
    |> recycle() # Recycle the connection to persist session data
  end

  describe "favorites functionality" do
    setup [:create_users, :create_advertisement]

    test "displays 'Add to Favorites' button for logged-in user on homepage", %{conn: conn, other_user: other_user, advertisement: advertisement} do
      conn = login_user(conn, other_user)
      conn = get(conn, "/") # Assuming the homepage is at "/home"

      # Check if the 'Add to Favorites' button is displayed
      assert html_response(conn, 200) =~ "Add to Favorites"
    end



    test "successfully toggles favorite status from homepage", %{conn: conn, user: user, advertisement: advertisement} do
      conn = login_user(conn, user)
      advertisement_id = advertisement.id

      # Simulate the toggle favorite request
      conn = post(conn, ~p"/advertisements/#{advertisement_id}/toggle_favorite", %{is_favorite: false})

      # Check the response for success and updated favorite status
      assert json_response(conn, 200)["status"] == "success"

      # Reload the user to check the updated favorite list
      updated_user = Repo.get(User, user.id)
      assert advertisement_id in updated_user.favorite_advertisements
    end

    test "prevents unauthenticated users from toggling favorites", %{conn: conn, advertisement: advertisement} do
      conn = post(conn, ~p"/advertisements/#{advertisement.id}/toggle_favorite", %{is_favorite: false})
      assert json_response(conn, 401)["status"] == "error"
    end
  end
end
