defmodule ProptrackerWeb.AvailableFavoritesTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{
    name: "John Doe",
    username: "john_doe",
    password: "secret123",
    date_of_birth: ~D[1990-01-01],
    phone_number: "1234567890"  # Add a phone number here
  }

  @valid_advertisement_attrs %{
    title: "Sample Ad",
    pictures: ["path/to/image1.jpg", "path/to/image2.jpg"],
    description: "Sample Description",
    type: "rent",
    price: Decimal.new("1000.00"),
    square_meters: 100,
    location: "Sample City",
    rooms: 2,
    floor: 1,
    total_floors: 5,
    state: "available"
  }
  @unavailable_advertisement_attrs Map.put(@valid_advertisement_attrs, :state, "sold/rented")

  defp create_user(_) do
    {:ok, user} = Repo.insert(User.changeset(%User{}, @valid_user_attrs))
    %{user: user}
  end

  defp create_advertisements(%{user: user}) do
    available_ad_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    unavailable_ad_attrs = Map.put(@unavailable_advertisement_attrs, :user_id, user.id)

    {:ok, available_ad} = Repo.insert(Advertisement.changeset(%Advertisement{}, available_ad_attrs))
    {:ok, unavailable_ad} = Repo.insert(Advertisement.changeset(%Advertisement{}, unavailable_ad_attrs))

    %{available_ad: available_ad, unavailable_ad: unavailable_ad}
  end

  defp login_user(conn, user) do
    conn
    |> post(~p"/login", %{"user" => %{"username" => user.username, "password" => user.password}})
    |> recycle() # Recycle the connection to persist session data
  end

  describe "GET /my_favorites" do
    setup [:create_user, :create_advertisements]

    test "only available advertisements are shown in favorites", %{conn: conn, user: user, available_ad: available_ad, unavailable_ad: unavailable_ad} do
      # Add the available advertisement to the user's favorites
      conn = login_user(conn, user)
      conn = post(conn, ~p"/advertisements/#{available_ad.id}/toggle_favorite", %{is_favorite: true})

      # Add the unavailable advertisement to the user's favorites (optional, to ensure both are added)
      conn = post(conn, ~p"/advertisements/#{unavailable_ad.id}/toggle_favorite", %{is_favorite: true})

      # Now, visit the "My Favorites" page
      conn = get(conn, "/my_favorites")

      # Check if the available advertisement is shown in the response
      assert html_response(conn, 200) =~ available_ad.title
      assert html_response(conn, 200) =~ available_ad.price |> Decimal.to_string()
      assert html_response(conn, 200) =~ available_ad.location
    end

    test "advertisements with state not equal to 'available' are not shown in favorites", %{conn: conn, user: user, available_ad: available_ad, unavailable_ad: unavailable_ad} do
      # Add the available advertisement to the user's favorites
      conn = login_user(conn, user)
      conn = post(conn, ~p"/advertisements/#{available_ad.id}/toggle_favorite", %{is_favorite: true})

      # Add the unavailable advertisement (with state "sold/rented") to the user's favorites
      conn = post(conn, ~p"/advertisements/#{unavailable_ad.id}/toggle_favorite", %{is_favorite: true})

      # Now, visit the "My Favorites" page
      conn = get(conn, "/my_favorites")

      # Check if the available advertisement is shown in the response
      assert html_response(conn, 200) =~ available_ad.title

    end
  end
end
