defmodule ProptrackerWeb.SearchControllerTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, SearchHistory}
  alias Proptracker.Repo

  # Valid user attributes
  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01]}

  # Valid search history attributes
  @valid_search_history_attrs %{
    type: "rent",
    location: "Tallinn",
    query: "apartment",
    inserted_at: ~N[2024-01-01 10:00:00]
  }

  # Create user helper function
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Create search history for a user
  defp create_search_history(%{user: user}) do
    search_history_attrs = Map.put(@valid_search_history_attrs, :user_id, user.id)

    {:ok, search_history} =
      %SearchHistory{}
      |> SearchHistory.changeset(search_history_attrs)
      |> Repo.insert()

    %{search_history: search_history}
  end

  # Login helper function
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "past search histories" do
    setup [:create_user, :create_search_history]

    test "renders past searches for authenticated user", %{conn: conn, user: user, search_history: search_history} do
      # Simulate logging in the user
      conn = login_user(conn, user)

      # Verify user is logged in
      assert get_session(conn, :user_id) == user.id

      # Simulate GET request to past_searches route
      conn = get(conn, "/past_searches")

      # Assert that the past search history is shown
      assert html_response(conn, 200) =~ "Past Searches"
      assert html_response(conn, 200) =~ search_history.query
      assert html_response(conn, 200) =~ search_history.location
      assert html_response(conn, 200) =~ search_history.type
    end

    test "redirects to login page for unauthenticated user", %{conn: conn} do
      # Simulate GET request to past_searches without logging in
      conn = get(conn, "/past_searches")

      # Assert redirection to the login page
      assert redirected_to(conn) == "/login"
      assert get_flash(conn, :error) == "You need to log in to access past searches."
    end
  end
end
