defmodule ProptrackerWeb.AdvertisementGoogleMapTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01], phone_number: "123456789"}
  @valid_advertisement_attrs %{
    title: "Beautiful House",
    description: "A beautiful house for sale",
    price: 200000,
    location: "Downtown",
    square_meters: 150,
    rooms: 4,
    floor: 2,
    total_floors: 3,
    state: "available",
    reference: "REF123",
    pictures: ["/path/to/image1.jpg"],
    latitude: 52.370216,
    longitude: 4.895168,
    type: "sell"
  }
  @invalid_advertisement_attrs %{
    title: "No Map House",
    description: "No map available for this house",
    price: 150000,
    location: "Uptown",
    square_meters: 120,
    rooms: 3,
    floor: 1,
    total_floors: 2,
    state: "sold",
    reference: "REF124",
    pictures: ["/path/to/image2.jpg"],
    latitude: nil,
    longitude: nil,
    type: "sell"
  }

  # Helper function to create a user
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Helper function to create advertisements
  defp create_advertisement(%{user: user}) do
    valid_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    invalid_attrs = Map.put(@invalid_advertisement_attrs, :user_id, user.id)

    {:ok, valid_advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, valid_attrs))
    {:ok, invalid_advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, invalid_attrs))

    %{valid_advertisement: Repo.preload(valid_advertisement, :user), invalid_advertisement: Repo.preload(invalid_advertisement, :user)}
  end

  # Helper function to log in a user
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "Google Map functionality" do
    setup [:create_user, :create_advertisement]

    test "renders map when latitude and longitude are present", %{conn: conn, valid_advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure the user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Access the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Check if the map is rendered
      assert html_response(conn, 200) =~ "<div id=\"map\""
      assert html_response(conn, 200) =~ "initMap()"
    end

    test "renders fallback message when latitude and longitude are absent", %{conn: conn, invalid_advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure the user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Access the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Check for fallback message
      assert html_response(conn, 200) =~ "No map available for this advertisement"
      refute html_response(conn, 200) =~ "<div id=\"map\""
    end
  end
end
