defmodule ProptrackerWeb.AdvertisementSocialMediaTest do
  use ProptrackerWeb.ConnCase

  alias Proptracker.Accounts.{User, Advertisement}
  alias Proptracker.Repo

  # Define valid attributes for creating a user and an advertisement
  @valid_user_attrs %{name: "John Doe", username: "john_doe", password: "secret123", date_of_birth: ~D[1990-01-01], phone_number: "123456789"}
  @valid_advertisement_attrs %{
    title: "Beautiful House",
    description: "A beautiful house for sale",
    price: 200000,
    location: "Downtown",
    square_meters: 150,
    rooms: 4,
    floor: 2,
    total_floors: 3,
    state: "available",
    reference: "REF123",
    pictures: ["/path/to/image1.jpg"],
    type: "sell"
  }

  # Helper function to create a user
  defp create_user(_) do
    {:ok, user} = Repo.insert(%User{} |> User.changeset(@valid_user_attrs))
    %{user: user}
  end

  # Helper function to create an advertisement
  defp create_advertisement(%{user: user}) do
    valid_attrs = Map.put(@valid_advertisement_attrs, :user_id, user.id)
    {:ok, valid_advertisement} = Repo.insert(Advertisement.changeset(%Advertisement{}, valid_attrs))
    %{valid_advertisement: Repo.preload(valid_advertisement, :user)}
  end

  # Helper function to simulate user login
  defp login_user(conn, user) do
    post(conn, "/login", user: %{username: user.username, password: user.password})
  end

  describe "Social Media functionality" do
    setup [:create_user, :create_advertisement]

    test "displays social media sharing buttons on the advertisement page", %{conn: conn, valid_advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure the user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Access the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Check for social media sharing buttons
      assert html_response(conn, 200) =~ "Share on Facebook"
    end
    test "includes correct sharing URLs for the advertisement", %{conn: conn, valid_advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure the user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Access the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")

      # Construct expected sharing URLs
      expected_url = ~s(#{ProptrackerWeb.Endpoint.url()}/advertisements/#{advertisement.id})

      # Check if the correct sharing URL is included in the page
      assert html_response(conn, 200) =~ expected_url
    end
    test "does not display social media sharing buttons on the advertisement page", %{conn: conn, valid_advertisement: advertisement} do
      # Simulate logging in the user
      conn = login_user(conn, advertisement.user)

      # Ensure the user is logged in
      assert get_session(conn, :user_id) == advertisement.user.id

      # Access the advertisement's show page
      conn = get(conn, ~p"/advertisements/#{advertisement.id}")
   end
  end
end
